#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
Test class for iRMC Power Driver
"""

import builtins
import io
import os
import time
from unittest import mock
import xml.etree.ElementTree as ET

import defusedxml.ElementTree as dET
from requests_mock.contrib import fixture as rm_fixture
import testtools

from scciclient.irmc import ipmi
from scciclient.irmc import scci
from scciclient.irmc import snmp


IRMC_CONFIG_PARTIAL = """<?xml version="1.0" encoding="UTF-8" standalone="yes" ?>
<CMDSEQ>
<!-- "Many Items Ommited..." -->
<!-- "ConfBmcRemoteFdImageServer" -->
<CMD Context="SCCI" OC="ConfigSpace" OE="1A50" OI="0" Type="SET">
  <DATA Type="xsd::string">10.124.196.156</DATA>
  <STATUS>0</STATUS>
</CMD>
<!-- "ConfBmcRemoteFdImageUserName" -->
<CMD Context="SCCI" OC="ConfigSpace" OE="1A51" OI="0" Type="SET">
  <DATA Type="xsd::string"></DATA>
  <STATUS>0</STATUS>
</CMD>
<!-- "ConfBmcRemoteFdImageUserPassword" -->
<CMD Context="SCCI" OC="ConfigSpace" OE="1A52" OI="0" Type="SET">
  <DATA Type="xsd::string"></DATA>
  <STATUS>0</STATUS>
</CMD>
<!-- "ConfBmcRemoteFdImageUserDomain" -->
<CMD Context="SCCI" OC="ConfigSpace" OE="1A53" OI="0" Type="SET">
  <DATA Type="xsd::string"></DATA>
  <STATUS>0</STATUS>
</CMD>
<!-- "ConfBmcRemoteFdImageShareType" -->
<CMD Context="SCCI" OC="ConfigSpace" OE="1A54" OI="0" Type="SET">
  <DATA Type="xsd::integer">1</DATA>
  <STATUS>0</STATUS>
</CMD>
<!-- "ConfBmcRemoteFdImageShareName" -->
<CMD Context="SCCI" OC="ConfigSpace" OE="1A55" OI="0" Type="SET">
  <DATA Type="xsd::string">/srv/nfs</DATA>
  <STATUS>0</STATUS>
</CMD>
<!-- "ConfBmcRemoteFdImageImageName" -->
<CMD Context="SCCI" OC="ConfigSpace" OE="1A56" OI="0" Type="SET">
  <DATA Type="xsd::string">virtual-media.img</DATA>
  <STATUS>0</STATUS>
</CMD>
<!-- "ConfBmcMediaOptionsFdAttachMode" -->
<CMD Context="SCCI" OC="ConfigSpace" OE="1A57" OI="0" Type="SET">
  <DATA Type="xsd::integer">1</DATA>
  <STATUS>0</STATUS>
</CMD>
<!-- "ConfBmcMediaOptionsFdNumber" -->
<CMD Context="SCCI" OC="ConfigSpace" OE="1A58" OI="0" Type="SET">
  <DATA Type="xsd::integer">1</DATA>
  <STATUS>0</STATUS>
</CMD>
<!-- "ConfBmcRemoteCdImageServer" -->
<CMD Context="SCCI" OC="ConfigSpace" OE="1A60" OI="0" Type="SET">
  <DATA Type="xsd::string">10.124.196.156</DATA>
  <STATUS>0</STATUS>
</CMD>
<!-- "ConfBmcRemoteCdImageUserName" -->
<CMD Context="SCCI" OC="ConfigSpace" OE="1A61" OI="0" Type="SET">
  <DATA Type="xsd::string"></DATA>
  <STATUS>0</STATUS>
</CMD>
<!-- "ConfBmcRemoteCdImageUserPassword" -->
<CMD Context="SCCI" OC="ConfigSpace" OE="1A62" OI="0" Type="SET">
  <DATA Type="xsd::string"></DATA>
  <STATUS>0</STATUS>
</CMD>
<!-- "ConfBmcRemoteCdImageUserDomain" -->
<CMD Context="SCCI" OC="ConfigSpace" OE="1A63" OI="0" Type="SET">
  <DATA Type="xsd::string"></DATA>
  <STATUS>0</STATUS>
</CMD>
<!-- "ConfBmcRemoteCdImageShareType" -->
<CMD Context="SCCI" OC="ConfigSpace" OE="1A64" OI="0" Type="SET">
  <DATA Type="xsd::integer">1</DATA>
  <STATUS>0</STATUS>
</CMD>
<!-- "ConfBmcRemoteCdImageShareName" -->
<CMD Context="SCCI" OC="ConfigSpace" OE="1A65" OI="0" Type="SET">
  <DATA Type="xsd::string">/srv/nfs</DATA>
  <STATUS>0</STATUS>
</CMD>
<!-- "ConfBmcRemoteCdImageImageName" -->
<CMD Context="SCCI" OC="ConfigSpace" OE="1A66" OI="0" Type="SET">
  <DATA Type="xsd::string">virtual-media.iso</DATA>
  <STATUS>0</STATUS>
</CMD>
<!-- "ConfBmcMediaOptionsCdAttachMode" -->
<CMD Context="SCCI" OC="ConfigSpace" OE="1A67" OI="0" Type="SET">
  <DATA Type="xsd::integer">1</DATA>
  <STATUS>0</STATUS>
</CMD>
<!-- "ConfBmcMediaOptionsCdNumber" -->
<CMD Context="SCCI" OC="ConfigSpace" OE="1A68" OI="0" Type="SET">
  <DATA Type="xsd::integer">1</DATA>
  <STATUS>0</STATUS>
</CMD>
<!-- "Many Items Ommited..." -->
</CMDSEQ>"""  # noqa: E501


class SCCITestCase(testtools.TestCase):
    """Tests for SCCI

    Unit Test Cases for power on/off/reset, and mount cd/fd
    """

    def setUp(self):
        super(SCCITestCase, self).setUp()

        self.requests_mock = self.useFixture(rm_fixture.Fixture())

        with open(os.path.join(
                os.path.dirname(__file__),
                'fixtures/irmc_report_ok.xml'), "r") as report_ok:
            self.report_ok_txt = report_ok.read()
        self.report_ok_xml = dET.fromstring(self.report_ok_txt)

        with open(os.path.join(
                os.path.dirname(__file__),
                'fixtures/irmc_report_ng.xml'), "r") as report_ng:
            self.report_ng_txt = report_ng.read()
        self.report_ng_xml = dET.fromstring(self.report_ng_txt)

        self.irmc_address = '10.124.196.159'
        self.irmc_username = 'admin'
        self.irmc_password = 'admin0'
        self.irmc_port = 80
        self.irmc_auth_method = 'basic'
        self.irmc_client_timeout = 60
        self.irmc_info = {'irmc_address': self.irmc_address,
                          'irmc_username': self.irmc_username,
                          'irmc_password': self.irmc_password,
                          'irmc_snmp_port': 161,
                          'irmc_snmp_version': 'v2c',
                          'irmc_snmp_community': 'public',
                          'irmc_snmp_security': None,
                          'irmc_client_timeout': self.irmc_client_timeout,
                          'irmc_sensor_method': 'ipmitool',
                          'irmc_auth_method': self.irmc_auth_method,
                          'irmc_port': 443,
                          'irmc_tempdir': "/tmp"
                          }

        self.irmc_remote_image_server = '10.33.110.49'
        self.irmc_remote_image_user_domain = 'example.local'
        self.irmc_remote_image_share_type = scci.ShareType.nfs
        self.irmc_remote_image_share_name = 'share'
        self.irmc_remote_image_deploy_iso = 'ubuntu-14.04.1-server-amd64.iso'
        self.irmc_remote_image_username = 'deployer'
        self.irmc_remote_image_user_password = 'password'

    def test_get_share_type_ok(self):
        nfs_result = scci.get_share_type("nfs")
        self.assertEqual(scci.ShareType.nfs, nfs_result)
        cifs_result = scci.get_share_type("cifs")
        self.assertEqual(scci.ShareType.cifs, cifs_result)

        NFS_result = scci.get_share_type("NFS")
        self.assertEqual(scci.ShareType.nfs, NFS_result)
        CIFS_result = scci.get_share_type("CIFS")
        self.assertEqual(scci.ShareType.cifs, CIFS_result)

    def test_get_share_type_ng(self):
        self.assertRaises(KeyError,
                          scci.get_share_type,
                          "abc")

    @mock.patch('scciclient.irmc.scci.requests')
    def test_scci_cmd_protocol_https_ok(self, mock_requests):
        https_port = 443
        mock_requests.post.return_value = mock.Mock(
            return_value='ok',
            status_code=200,
            text="""<?xml version="1.0" encoding="UTF-8"?>
            <Status>
            <Value>0</Value>
            <Severity>Information</Severity>
            <Message>No Error</Message>
            </Status>""")
        returned_mock_requests_post = scci.scci_cmd(
            self.irmc_address,
            self.irmc_username,
            self.irmc_password,
            scci.POWER_ON,
            port=https_port,
            auth_method=self.irmc_auth_method,
            client_timeout=self.irmc_client_timeout)
        mock_requests.post.assert_called_with(
            'https://' + self.irmc_address + '/config',
            data=scci.POWER_ON,
            headers={'Content-type': 'application/x-www-form-urlencoded'},
            verify=False,
            timeout=self.irmc_client_timeout,
            allow_redirects=False,
            auth=mock_requests.auth.HTTPBasicAuth(self.irmc_username,
                                                  self.irmc_password))
        self.assertEqual('ok', returned_mock_requests_post.return_value)

    def test_scci_cmd_protocol_http_and_auth_basic_ok(self):
        self.requests_mock.post("http://" + self.irmc_address + "/config",
                                text="""<?xml version="1.0" encoding="UTF-8"?>
                                        <Status>
                                            <Value>0</Value>
                                            <Severity>Information</Severity>
                                            <Message>No Error</Message>
                                        </Status>""")

        r = scci.scci_cmd(self.irmc_address,
                          self.irmc_username,
                          self.irmc_password,
                          scci.POWER_ON,
                          port=self.irmc_port,
                          auth_method=self.irmc_auth_method,
                          client_timeout=self.irmc_client_timeout)
        self.assertEqual(r.status_code, 200)

    def test_scci_cmd_protocol_http_and_auth_digest_ok(self):
        self.requests_mock.post("http://" + self.irmc_address + "/config",
                                text="""<?xml version="1.0" encoding="UTF-8"?>
                                     <Status>
                                         <Value>0</Value>
                                         <Severity>Information</Severity>
                                         <Message>No Error</Message>
                                     </Status>""")

        auth_digest = 'digest'
        r = scci.scci_cmd(self.irmc_address,
                          self.irmc_username,
                          self.irmc_password,
                          scci.POWER_ON,
                          port=self.irmc_port,
                          auth_method=auth_digest,
                          client_timeout=self.irmc_client_timeout)
        self.assertEqual(r.status_code, 200)

    def test_scci_cmd_authentication_failure(self):
        self.requests_mock.post("http://" + self.irmc_address + "/config",
                                text="401 Unauthorized",
                                status_code=401)

        e = self.assertRaises(scci.SCCIClientError,
                              scci.scci_cmd,
                              self.irmc_address,
                              self.irmc_username,
                              self.irmc_password,
                              scci.POWER_ON,
                              port=self.irmc_port,
                              auth_method=self.irmc_auth_method,
                              client_timeout=self.irmc_client_timeout)

        self.assertEqual(
            'HTTP PROTOCOL ERROR, STATUS CODE = 401',
            str(e))

    def test_scci_cmd_protocol_ng(self):
        ssh_port = 22
        e = self.assertRaises(scci.SCCIInvalidInputError,
                              scci.scci_cmd,
                              self.irmc_address,
                              self.irmc_username,
                              self.irmc_password,
                              scci.POWER_ON,
                              port=ssh_port,
                              auth_method=self.irmc_auth_method,
                              client_timeout=self.irmc_client_timeout)
        self.assertEqual((("Invalid port %(port)d or "
                           "auth_method for method %(auth_method)s") %
                          {'port': ssh_port,
                           'auth_method': self.irmc_auth_method}), str(e))

    def test_scci_cmd_auth_method_ng(self):
        unknown_auth_method = 'unknown'
        e = self.assertRaises(scci.SCCIInvalidInputError,
                              scci.scci_cmd,
                              self.irmc_address,
                              self.irmc_username,
                              self.irmc_password,
                              scci.POWER_ON,
                              port=self.irmc_port,
                              auth_method=unknown_auth_method,
                              client_timeout=self.irmc_client_timeout)
        self.assertEqual(("Invalid port %(port)d or "
                          "auth_method for method %(auth_method)s") %
                         {'port': self.irmc_port,
                          'auth_method': unknown_auth_method}, str(e))

    def test_power_on_scci_xml_parse_failed(self):
        self.requests_mock.post(
            "http://" + self.irmc_address + "/config",
            text="""<?xml version="1.0" encoding="UTF-8"?>
            <Status>
            <Value>31</Value>
            <Severity>Error</Severity>
            <Message>Error 31 (Import of settings in WinSCU XML format failed)
            occurred</Message>
            <Error Context="SCCI" OC="0" OE="0" OI="0">
            XML parser creation failed (Error parsing:
            attribute value should start with a quote
            SEQ>  <CMD Context="SCCI" OC=PowerOnCabinet OE="0" OI="0"
            -----------------------------^------------------------------).
            </Error>
            </Status>""")

        client = scci.get_client(self.irmc_address,
                                 self.irmc_username,
                                 self.irmc_password,
                                 port=self.irmc_port,
                                 auth_method=self.irmc_auth_method,
                                 client_timeout=self.irmc_client_timeout)

        e = self.assertRaises(scci.SCCIClientError,
                              client,
                              scci.POWER_ON)
        self.assertEqual(
            'not well-formed (invalid token): line 10, column 41',
            str(e))

    def test_power_on_http_failed(self):
        self.requests_mock.post("http://" + self.irmc_address + "/config",
                                text="anything",
                                status_code=302)

        client = scci.get_client(self.irmc_address,
                                 self.irmc_username,
                                 self.irmc_password,
                                 port=self.irmc_port,
                                 auth_method=self.irmc_auth_method,
                                 client_timeout=self.irmc_client_timeout)

        e = self.assertRaises(scci.SCCIClientError,
                              client,
                              scci.POWER_ON)
        self.assertEqual(
            'HTTP PROTOCOL ERROR, STATUS CODE = 302',
            str(e))

    @mock.patch.object(time, 'sleep')
    def test_power_on_ok(self, mock_sleep):
        self.requests_mock.post("http://" + self.irmc_address + "/config",
                                text="""<?xml version="1.0" encoding="UTF-8"?>
                                <Status>
                                    <Value>0</Value>
                                    <Severity>Information</Severity>
                                    <Message>No Error</Message>
                                </Status>""")

        client = scci.get_client(self.irmc_address,
                                 self.irmc_username,
                                 self.irmc_password,
                                 port=self.irmc_port,
                                 auth_method=self.irmc_auth_method,
                                 client_timeout=self.irmc_client_timeout)
        r = client(scci.POWER_ON)
        self.assertEqual(r.status_code, 200)
        mock_sleep.assert_called_once_with(3)

    @mock.patch.object(time, 'sleep')
    def test_power_off_ok(self, mock_sleep):
        self.requests_mock.post("http://" + self.irmc_address + "/config",
                                text="""<?xml version="1.0" encoding="UTF-8"?>
                                <Status>
                                    <Value>0</Value>
                                    <Severity>Information</Severity>
                                    <Message>No Error</Message>
                                </Status>""")

        client = scci.get_client(self.irmc_address,
                                 self.irmc_username,
                                 self.irmc_password,
                                 port=self.irmc_port,
                                 auth_method=self.irmc_auth_method,
                                 client_timeout=self.irmc_client_timeout)
        r = client(scci.POWER_OFF)
        self.assertEqual(r.status_code, 200)
        mock_sleep.assert_called_once_with(3)

    def test_power_cycle_ok(self):
        self.requests_mock.post("http://" + self.irmc_address + "/config",
                                text="""<?xml version="1.0" encoding="UTF-8"?>
                                <Status>
                                    <Value>0</Value>
                                    <Severity>Information</Severity>
                                    <Message>No Error</Message>
                                </Status>""")

        client = scci.get_client(self.irmc_address,
                                 self.irmc_username,
                                 self.irmc_password,
                                 port=self.irmc_port,
                                 auth_method=self.irmc_auth_method,
                                 client_timeout=self.irmc_client_timeout)
        r = client(scci.POWER_CYCLE)
        self.assertEqual(r.status_code, 200)

    @mock.patch.object(time, 'sleep')
    def test_power_reset_ok(self, mock_sleep):
        self.requests_mock.post("http://" + self.irmc_address + "/config",
                                text="""<?xml version="1.0" encoding="UTF-8"?>
                                <Status>
                                    <Value>0</Value>
                                    <Severity>Information</Severity>
                                    <Message>No Error</Message>
                                </Status>""")

        client = scci.get_client(self.irmc_address,
                                 self.irmc_username,
                                 self.irmc_password,
                                 port=self.irmc_port,
                                 auth_method=self.irmc_auth_method,
                                 client_timeout=self.irmc_client_timeout)
        r = client(scci.POWER_RESET)
        self.assertEqual(r.status_code, 200)
        mock_sleep.assert_called_once_with(3)

    def test_power_raise_nmi_ok(self):
        self.requests_mock.post("http://" + self.irmc_address + "/config",
                                text="""<?xml version="1.0" encoding="UTF-8"?>
                                <Status>
                                    <Value>0</Value>
                                    <Severity>Information</Severity>
                                    <Message>No Error</Message>
                                </Status>""")

        client = scci.get_client(self.irmc_address,
                                 self.irmc_username,
                                 self.irmc_password,
                                 port=self.irmc_port,
                                 auth_method=self.irmc_auth_method,
                                 client_timeout=self.irmc_client_timeout)
        r = client(scci.POWER_RAISE_NMI)
        self.assertEqual(r.status_code, 200)

    def test_power_soft_off_ok(self):
        self.requests_mock.post("http://" + self.irmc_address + "/config",
                                text="""<?xml version="1.0" encoding="UTF-8"?>
                                <Status>
                                    <Value>0</Value>
                                    <Severity>Information</Severity>
                                    <Message>No Error</Message>
                                </Status>""")

        client = scci.get_client(self.irmc_address,
                                 self.irmc_username,
                                 self.irmc_password,
                                 port=self.irmc_port,
                                 auth_method=self.irmc_auth_method,
                                 client_timeout=self.irmc_client_timeout)
        r = client(scci.POWER_SOFT_OFF)
        self.assertEqual(r.status_code, 200)

    def test_power_soft_off_ng(self):
        self.requests_mock.post(
            "http://" + self.irmc_address + "/config",
            text="""<?xml version="1.0" encoding="UTF-8"?>
            <Status>
            <Value>31</Value>
            <Severity>Error</Severity>
            <Message>Error 31 (Import of settings in WinSCU"""
            """ XML format failed) occurred</Message>
            <Error Context="SCCI" OC="ShutdownRequestCancelled"
             OE="0" OI="0">ServerView Agent not connected</Error>
            </Status>""")

        client = scci.get_client(self.irmc_address,
                                 self.irmc_username,
                                 self.irmc_password,
                                 port=self.irmc_port,
                                 auth_method=self.irmc_auth_method,
                                 client_timeout=self.irmc_client_timeout)
        e = self.assertRaises(scci.SCCIClientError,
                              client,
                              scci.POWER_SOFT_OFF)
        self.assertEqual(
            'SCCI PROTOCOL ERROR, STATUS CODE = 31,'
            ' ERROR = ServerView Agent not connected, MESSAGE = Error 31'
            ' (Import of settings in WinSCU XML format failed) occurred',
            str(e))

    def test_power_soft_cycle_ok(self):
        self.requests_mock.post("http://" + self.irmc_address + "/config",
                                text="""<?xml version="1.0" encoding="UTF-8"?>
                                <Status>
                                    <Value>0</Value>
                                    <Severity>Information</Severity>
                                    <Message>No Error</Message>
                                </Status>""")

        client = scci.get_client(self.irmc_address,
                                 self.irmc_username,
                                 self.irmc_password,
                                 port=self.irmc_port,
                                 auth_method=self.irmc_auth_method,
                                 client_timeout=self.irmc_client_timeout)
        r = client(scci.POWER_SOFT_CYCLE)
        self.assertEqual(r.status_code, 200)

    def test_power_soft_cycle_ng(self):
        self.requests_mock.post(
            "http://" + self.irmc_address + "/config",
            text="""<?xml version="1.0" encoding="UTF-8"?>
            <Status>
            <Value>31</Value>
            <Severity>Error</Severity>
            <Message>Error 31 (Import of settings in WinSCU"""
            """ XML format failed) occurred</Message>
            <Error Context="SCCI" OC="ShutdownRequestCancelled"
             OE="0" OI="0">ServerView Agent not connected</Error>
            </Status>""")

        client = scci.get_client(self.irmc_address,
                                 self.irmc_username,
                                 self.irmc_password,
                                 port=self.irmc_port,
                                 auth_method=self.irmc_auth_method,
                                 client_timeout=self.irmc_client_timeout)
        e = self.assertRaises(scci.SCCIClientError,
                              client,
                              scci.POWER_SOFT_CYCLE)
        self.assertEqual(
            'SCCI PROTOCOL ERROR, STATUS CODE = 31,'
            ' ERROR = ServerView Agent not connected, MESSAGE = Error 31'
            ' (Import of settings in WinSCU XML format failed) occurred',
            str(e))

    def test_power_cancel_shutdown_ok(self):
        self.requests_mock.post("http://" + self.irmc_address + "/config",
                                text="""<?xml version="1.0" encoding="UTF-8"?>
                                <Status>
                                    <Value>0</Value>
                                    <Severity>Information</Severity>
                                    <Message>No Error</Message>
                                </Status>""")

        client = scci.get_client(self.irmc_address,
                                 self.irmc_username,
                                 self.irmc_password,
                                 port=self.irmc_port,
                                 auth_method=self.irmc_auth_method,
                                 client_timeout=self.irmc_client_timeout)
        r = client(scci.POWER_CANCEL_SHUTDOWN)
        self.assertEqual(r.status_code, 200)

    def test_power_cancel_shutdown_ng(self):
        self.requests_mock.post(
            "http://" + self.irmc_address + "/config",
            text="""<?xml version="1.0" encoding="UTF-8"?>
            <Status>
            <Value>31</Value>
            <Severity>Error</Severity>
            <Message>Error 31 (Import of settings in WinSCU"""
            """ XML format failed) occurred</Message>
            <Error Context="SCCI" OC="ShutdownRequestCancelled"
             OE="0" OI="0">ServerView Agent not connected</Error>
            </Status>""")

        client = scci.get_client(self.irmc_address,
                                 self.irmc_username,
                                 self.irmc_password,
                                 port=self.irmc_port,
                                 auth_method=self.irmc_auth_method,
                                 client_timeout=self.irmc_client_timeout)
        e = self.assertRaises(scci.SCCIClientError,
                              client,
                              scci.POWER_CANCEL_SHUTDOWN)
        self.assertEqual(
            'SCCI PROTOCOL ERROR, STATUS CODE = 31,'
            ' ERROR = ServerView Agent not connected, MESSAGE = Error 31'
            ' (Import of settings in WinSCU XML format failed) occurred',
            str(e))

    def test_get_sensor_data_records_ok(self):
        sensor = scci.get_sensor_data_records(self.report_ok_xml)
        self.assertEqual(len(sensor), 10)

    def test_get_sensor_data_records_ng(self):
        sensor = scci.get_sensor_data_records(self.report_ng_xml)
        self.assertIsNone(sensor)

    def test_get_irmc_version_ok(self):
        version = scci.get_irmc_version(self.report_ok_xml)
        self.assertEqual(version.attrib['Name'], "iRMC S4")

    def test_get_irmc_version_ng(self):
        version = scci.get_irmc_version(self.report_ng_xml)
        self.assertIsNone(version)

    def test_get_report_ok(self):
        self.requests_mock.get(
            "http://" + self.irmc_address + "/report.xml",
            text=self.report_ok_txt,
            headers={'Content-Type': "application/x-www-form-urlencoded"})

        root = scci.get_report(self.irmc_address,
                               self.irmc_username,
                               self.irmc_password,
                               port=self.irmc_port,
                               auth_method=self.irmc_auth_method,
                               client_timeout=self.irmc_client_timeout)

        self.assertEqual(root.tag, 'Root')

        sensor = scci.get_sensor_data_records(root)
        self.assertEqual(sensor.tag, 'SensorDataRecords')

    def test_get_report_http_failed(self):
        self.requests_mock.get(
            "http://" + self.irmc_address + "/report.xml",
            text=self.report_ok_txt,
            headers={'Content-Type': "application/x-www-form-urlencoded"},
            status_code=302)

        e = self.assertRaises(scci.SCCIClientError,
                              scci.get_report,
                              self.irmc_address,
                              self.irmc_username,
                              self.irmc_password,
                              port=self.irmc_port,
                              auth_method=self.irmc_auth_method,
                              client_timeout=self.irmc_client_timeout)
        self.assertEqual(
            'HTTP PROTOCOL ERROR, STATUS CODE = 302',
            str(e))

    @mock.patch.object(time, 'sleep')
    def test_virtual_media_cd_setting_ok(self, sleep_mock):
        self.requests_mock.post("http://" + self.irmc_address + "/config",
                                text="""<?xml version="1.0" encoding="UTF-8"?>
                                <Status>
                                    <Value>0</Value>
                                    <Severity>Information</Severity>
                                    <Message>No Error</Message>
                                </Status>""")

        cmd = scci.get_virtual_cd_set_params_cmd(
            self.irmc_remote_image_server,
            self.irmc_remote_image_user_domain,
            self.irmc_remote_image_share_type,
            self.irmc_remote_image_share_name,
            self.irmc_remote_image_deploy_iso,
            self.irmc_remote_image_username,
            self.irmc_remote_image_user_password)
        client = scci.get_client(self.irmc_address,
                                 self.irmc_username,
                                 self.irmc_password,
                                 port=self.irmc_port,
                                 auth_method=self.irmc_auth_method,
                                 client_timeout=self.irmc_client_timeout)

        r = client(cmd, do_async=False)
        self.assertEqual(r.status_code, 200)
        sleep_mock.assert_called_once_with(5)

    @mock.patch.object(time, 'sleep')
    def test_virtual_media_fd_setting_ok(self, sleep_mock):
        self.requests_mock.post("http://" + self.irmc_address + "/config",
                                text="""<?xml version="1.0" encoding="UTF-8"?>
                                <Status>
                                    <Value>0</Value>
                                    <Severity>Information</Severity>
                                    <Message>No Error</Message>
                                </Status>""")

        cmd = scci.get_virtual_fd_set_params_cmd(
            self.irmc_remote_image_server,
            self.irmc_remote_image_user_domain,
            self.irmc_remote_image_share_type,
            self.irmc_remote_image_share_name,
            'floppy1.flp',
            self.irmc_remote_image_username,
            self.irmc_remote_image_user_password)
        client = scci.get_client(self.irmc_address,
                                 self.irmc_username,
                                 self.irmc_password,
                                 port=self.irmc_port,
                                 auth_method=self.irmc_auth_method,
                                 client_timeout=self.irmc_client_timeout)
        r = client(cmd, do_async=False)
        self.assertEqual(r.status_code, 200)
        sleep_mock.assert_called_once_with(5)

    @mock.patch.object(time, 'sleep')
    def test_mount_cd_ok(self, sleep_mock):
        self.requests_mock.post("http://" + self.irmc_address + "/config",
                                text="""<?xml version="1.0" encoding="UTF-8"?>
                                <Status>
                                    <Value>0</Value>
                                    <Severity>Information</Severity>
                                    <Message>No Error</Message>
                                </Status>""")

        client = scci.get_client(self.irmc_address,
                                 self.irmc_username,
                                 self.irmc_password,
                                 port=self.irmc_port,
                                 auth_method=self.irmc_auth_method,
                                 client_timeout=self.irmc_client_timeout)
        r = client(scci.MOUNT_CD)
        self.assertEqual(r.status_code, 200)
        self.assertFalse(sleep_mock.called)

    @mock.patch.object(time, 'sleep')
    def test_mount_fd_ok(self, sleep_mock):
        self.requests_mock.post("http://" + self.irmc_address + "/config",
                                text="""<?xml version="1.0" encoding="UTF-8"?>
                                <Status>
                                    <Value>0</Value>
                                    <Severity>Information</Severity>
                                    <Message>No Error</Message>
                                </Status>""")

        client = scci.get_client(self.irmc_address,
                                 self.irmc_username,
                                 self.irmc_password,
                                 port=self.irmc_port,
                                 auth_method=self.irmc_auth_method,
                                 client_timeout=self.irmc_client_timeout)
        r = client(scci.MOUNT_FD)
        self.assertEqual(r.status_code, 200)
        self.assertFalse(sleep_mock.called)

    def test_unmount_cd_ok(self):
        self.requests_mock.post("http://" + self.irmc_address + "/config",
                                text="""<?xml version="1.0" encoding="UTF-8"?>
                                <Status>
                                <Value>0</Value>
                                <Severity>Information</Severity>
                                <Message>No Error</Message>
                                </Status>""")

        self.requests_mock.get("http://" + self.irmc_address +
                               "/iRMC_Settings.pre", text=IRMC_CONFIG_PARTIAL)

        client = scci.get_client(self.irmc_address,
                                 self.irmc_username,
                                 self.irmc_password,
                                 port=self.irmc_port,
                                 auth_method=self.irmc_auth_method,
                                 client_timeout=self.irmc_client_timeout)
        r = client(scci.UNMOUNT_CD)
        self.assertEqual(r.status_code, 200)

    def test_unmount_fd_ok(self):
        self.requests_mock.post("http://" + self.irmc_address + "/config",
                                text="""<?xml version="1.0" encoding="UTF-8"?>
                                <Status>
                                    <Value>0</Value>
                                    <Severity>Information</Severity>
                                    <Message>No Error</Message>
                                </Status>""")

        client = scci.get_client(self.irmc_address,
                                 self.irmc_username,
                                 self.irmc_password,
                                 port=self.irmc_port,
                                 auth_method=self.irmc_auth_method,
                                 client_timeout=self.irmc_client_timeout)
        r = client(scci.MOUNT_FD)
        self.assertEqual(r.status_code, 200)

    def test_get_essential_properties(self):
        ESSENTIAL_PROPERTIES_KEYS = {
            'memory_mb', 'local_gb', 'cpus', 'cpu_arch'}
        expected = {'memory_mb': 8192,
                    'local_gb': 185,
                    'cpus': 16,
                    'cpu_arch': 'x86_64'}

        result = scci.get_essential_properties(
            self.report_ok_xml, ESSENTIAL_PROPERTIES_KEYS)

        self.assertEqual(expected, result)

    def test_get_essential_properties_empty_cpu_socket(self):
        ESSENTIAL_PROPERTIES_KEYS = {
            'memory_mb', 'local_gb', 'cpus', 'cpu_arch'}
        expected = {'memory_mb': 8192,
                    'local_gb': 185,
                    'cpus': 16,
                    'cpu_arch': 'x86_64'}

        result = scci.get_essential_properties(
            self.report_ng_xml, ESSENTIAL_PROPERTIES_KEYS)

        self.assertEqual(expected, result)

    @mock.patch.object(ipmi, 'get_pci_device')
    @mock.patch.object(snmp, 'get_server_model')
    @mock.patch.object(snmp, 'get_irmc_firmware_version')
    @mock.patch.object(snmp, 'get_bios_firmware_version')
    @mock.patch.object(ipmi, 'get_tpm_status')
    def test_get_capabilities_properties(self,
                                         tpm_mock,
                                         bios_mock,
                                         irmc_mock,
                                         server_mock,
                                         pci_device_mock):
        capabilities_properties = {'trusted_boot', 'irmc_firmware_version',
                                   'rom_firmware_version', 'server_model',
                                   'pci_gpu_devices'}
        gpu_ids = ['0x1000/0x0079', '0x2100/0x0080']
        kwargs = {}
        kwargs['sleep_flag'] = True

        tpm_mock.return_value = False
        bios_mock.return_value = 'V4.6.5.4 R1.15.0 for D3099-B1x'
        irmc_mock.return_value = 'iRMC S4-7.82F'
        server_mock.return_value = 'TX2540M1F5'
        pci_device_mock.side_effect = [1]

        expected = {'irmc_firmware_version': 'iRMC S4-7.82F',
                    'pci_gpu_devices': 1,
                    'rom_firmware_version': 'V4.6.5.4 R1.15.0 for D3099-B1x',
                    'server_model': 'TX2540M1F5',
                    'trusted_boot': False}

        result = scci.get_capabilities_properties(
            self.irmc_info,
            capabilities_properties,
            gpu_ids,
            **kwargs)

        self.assertEqual(expected, result)
        tpm_mock.assert_called_once_with(self.irmc_info)
        bios_mock.assert_called_once_with(mock.ANY)
        irmc_mock.assert_called_once_with(mock.ANY)
        server_mock.assert_called_once_with(mock.ANY)
        pci_device_mock.assert_called_once_with(self.irmc_info, gpu_ids)

    @mock.patch.object(ipmi, 'get_pci_device')
    @mock.patch.object(snmp, 'get_server_model')
    @mock.patch.object(snmp, 'get_irmc_firmware_version')
    @mock.patch.object(snmp, 'get_bios_firmware_version')
    @mock.patch.object(ipmi, 'get_tpm_status')
    def test_get_capabilities_properties_with_cpu_fpga(self,
                                                       tpm_mock,
                                                       bios_mock,
                                                       irmc_mock,
                                                       server_mock,
                                                       pci_device_mock):
        capabilities_properties = {'trusted_boot', 'irmc_firmware_version',
                                   'rom_firmware_version', 'server_model',
                                   'pci_gpu_devices', 'cpu_fpga'}
        gpu_ids = ['0x1000/0x0079', '0x2100/0x0080']
        cpu_fpgas = ['0x1000/0x0179', '0x2100/0x0180']
        kwargs = {}
        kwargs['sleep_flag'] = True

        tpm_mock.return_value = False
        bios_mock.return_value = 'V4.6.5.4 R1.15.0 for D3099-B1x'
        irmc_mock.return_value = 'iRMC S4-7.82F'
        server_mock.return_value = 'TX2540M1F5'
        pci_device_mock.side_effect = [1, 1]

        expected = {'irmc_firmware_version': 'iRMC S4-7.82F',
                    'pci_gpu_devices': 1,
                    'cpu_fpga': 1,
                    'rom_firmware_version': 'V4.6.5.4 R1.15.0 for D3099-B1x',
                    'server_model': 'TX2540M1F5',
                    'trusted_boot': False}

        result = scci.get_capabilities_properties(
            self.irmc_info,
            capabilities_properties,
            gpu_ids,
            cpu_fpgas,
            **kwargs)

        self.assertEqual(expected, result)
        tpm_mock.assert_called_once_with(self.irmc_info)
        bios_mock.assert_called_once_with(mock.ANY)
        irmc_mock.assert_called_once_with(mock.ANY)
        server_mock.assert_called_once_with(mock.ANY)
        pci_device_mock.assert_has_calls([
            mock.call(self.irmc_info, gpu_ids),
            mock.call(self.irmc_info, cpu_fpgas)])

    @mock.patch.object(ipmi, 'get_pci_device')
    @mock.patch.object(snmp, 'get_server_model')
    @mock.patch.object(snmp, 'get_irmc_firmware_version')
    @mock.patch.object(snmp, 'get_bios_firmware_version')
    @mock.patch.object(ipmi, 'get_tpm_status')
    def test_get_capabilities_properties_blank(self,
                                               tpm_mock,
                                               bios_mock,
                                               irmc_mock,
                                               server_mock,
                                               pci_device_mock):

        capabilities_properties = {}
        gpu_ids = ['0x1000/0x0079', '0x2100/0x0080']
        cpu_fpgas = ['0x1000/0x0179', '0x2100/0x0180']
        kwargs = {}
        kwargs['sleep_flag'] = True

        tpm_mock.return_value = False
        bios_mock.return_value = 'V4.6.5.4 R1.15.0 for D3099-B1x'
        irmc_mock.return_value = 'iRMC S4-7.82F'
        server_mock.return_value = 'TX2540M1F5'
        pci_device_mock.side_effect = [1, 1]

        expected = {}

        result = scci.get_capabilities_properties(
            self.irmc_info,
            capabilities_properties,
            gpu_ids,
            cpu_fpgas,
            **kwargs)

        self.assertEqual(expected, result)

    @mock.patch.object(ipmi, '_send_raw_command')
    @mock.patch.object(snmp.SNMPClient, 'get')
    def test_get_capabilities_properties_scci_client_error(self,
                                                           snmp_mock,
                                                           ipmiraw_mock):
        capabilities_properties = {'trusted_boot', 'irmc_firmware_version',
                                   'rom_firmware_version', 'server_model',
                                   'pci_gpu_devices', 'cpu_fpga'}
        gpu_ids = ['0x1000/0x0079', '0x2100/0x0080']
        cpu_fpgas = ['0x1000/0x0179', '0x2100/0x0180']
        kwargs = {}
        kwargs['sleep_flag'] = True

        ipmiraw_mock.return_value = None
        snmp_mock.side_effect = snmp.SNMPFailure("error")

        e = self.assertRaises(scci.SCCIClientError,
                              scci.get_capabilities_properties,
                              self.irmc_info,
                              capabilities_properties,
                              gpu_ids,
                              cpu_fpgas,
                              **kwargs)
        self.assertEqual('Capabilities inspection failed: SNMP operation \''
                         'GET BIOS FIRMWARE VERSION\' failed: error', str(e))

    @mock.patch.object(ipmi, 'get_pci_device')
    @mock.patch.object(snmp.SNMPClient, 'get')
    def test_get_capabilities_properties_scci_client_error_ipmi(self,
                                                                snmp_mock,
                                                                ipmi_mock):
        capabilities_properties = {'trusted_boot', 'irmc_firmware_version',
                                   'rom_firmware_version', 'server_model',
                                   'pci_gpu_devices', 'cpu_fpga'}
        gpu_ids = ['0x1000/0x0079', '0x2100/0x0080']
        cpu_fpgas = ['0x1000/0x0179', '0x2100/0x0180']
        kwargs = {}
        kwargs['sleep_flag'] = True

        ipmi_mock.side_effect = ipmi.IPMIFailure("IPMI error")
        snmp_mock.return_value = None

        e = self.assertRaises(scci.SCCIClientError,
                              scci.get_capabilities_properties,
                              self.irmc_info,
                              capabilities_properties,
                              gpu_ids,
                              cpu_fpgas,
                              **kwargs)
        self.assertEqual('Capabilities inspection failed: IPMI error', str(e))

    def test_fail_get_raid_fgi_status(self):
        report_fake = self.report_ok_xml
        report_fake.find("./Software/ServerView/ServerViewRaid").clear()
        self.assertRaises(scci.SCCIInvalidInputError,
                          scci.get_raid_fgi_status, report=report_fake)

    def test_fail_get_raid_fgi_status_1(self):
        report_fake = self.report_ok_xml
        report_fake.find("./Software/ServerView/ServerViewRaid/amEMSV"
                         "/System/Adapter").clear()
        self.assertRaises(scci.SCCIRAIDNotReady,
                          scci.get_raid_fgi_status, report=report_fake)

    def test_get_raid_fgi_status_ok(self):

        # Fake activity status of FGI in xml report
        url = "./Software/ServerView/ServerViewRaid/amEMSV/System/Adapter"
        report_fake = self.report_ok_xml
        report_input = report_fake.find(url)
        element_1 = ET.Element("LogicalDrive", name="LogicalDrive")
        element_2 = ET.Element("LogDriveNumber", name="LogDriveNumber")
        element_3 = ET.Element("Activity", name="Activity")
        report_input.append(element_1)
        report_input.find("./LogicalDrive").append(element_2)
        report_fake.find(url + "/LogicalDrive/LogDriveNumber").text = '0'
        report_input.find("./LogicalDrive").append(element_3)
        report_fake.find(url + "/LogicalDrive/Activity").text = 'Idle'

        fgi_status_expect = {'0': 'Idle'}
        result = scci.get_raid_fgi_status(report_fake)
        self.assertEqual(result, fgi_status_expect)

    @mock.patch('scciclient.irmc.scci.requests.get')
    def test_fail_get_bios_firmware_status(self, mock_requests_get):
        mock_requests_get.return_value = mock.Mock(
            status_code=404,
            text="""</head>
            <body>
            <div id="main">
                <div class="content message">
                    <div id="alert_icon">
                    </div>
                    <div id="msg_title" class="title">
                        File not found  <!-- This tag is OPTIONAL and is the
                        title eg. 404 - File not found -->
                    </div>
                </div>
            </div>
            </body>
            </html>""")
        upgrade_type = 'biosss'
        self.assertRaises(scci.SCCIClientError,
                          scci.get_firmware_upgrade_status, self.irmc_info,
                          upgrade_type=upgrade_type)

    @mock.patch('scciclient.irmc.scci.requests.get')
    def test_success_get_bios_firmware_status(self, mock_requests_get):
        mock_requests_get.return_value = mock.Mock(
            return_value='ok',
            status_code=200,
            text="""<?xml version="1.0" encoding="UTF-8"?>
            <Status>
            <Value>0</Value>
            <Severity>Information</Severity>
            <Message>No Error</Message>
            </Status>""")
        expected_status = "0"
        expected_severity = "Information"
        expected_message = "No Error"
        upgrade_type = 'bios'
        result = scci.get_firmware_upgrade_status(self.irmc_info, upgrade_type)
        self.assertEqual(expected_status, result.find("./Value").text)
        self.assertEqual(expected_severity, result.find("./Severity").text)
        self.assertEqual(expected_message, result.find("./Message").text)

    @mock.patch('scciclient.irmc.scci.requests.get')
    def test_fail_get_irmc_firmware_status(self, mock_requests_get):
        mock_requests_get.return_value = mock.Mock(
            status_code=404,
            text="""</head>
            <body>
            <div id="main">
                <div class="content message">
                    <div id="alert_icon">
                    </div>
                    <div id="msg_title" class="title">
                        File not found  <!-- This tag is OPTIONAL and is the
                        title eg. 404 - File not found -->
                    </div>
                </div>
            </div>
            </body>
            </html>""")
        upgrade_type = 'irmcccc'
        self.assertRaises(scci.SCCIClientError,
                          scci.get_firmware_upgrade_status, self.irmc_info,
                          upgrade_type=upgrade_type)

    @mock.patch('scciclient.irmc.scci.requests.get')
    def test_success_get_irmc_firmware_status(self, mock_requests_get):
        mock_requests_get.return_value = mock.Mock(
            return_value='ok',
            status_code=200,
            text="""<?xml version="1.0" encoding="UTF-8"?>
            <Status>
            <Value>0</Value>
            <Severity>Information</Severity>
            <Message>No Error</Message>
            </Status>""")
        expected_status = "0"
        expected_severity = "Information"
        expected_message = "No Error"
        upgrade_type = 'irmc'
        result = scci.get_firmware_upgrade_status(self.irmc_info, upgrade_type)
        self.assertEqual(expected_status, result.find("./Value").text)
        self.assertEqual(expected_severity, result.find("./Severity").text)
        self.assertEqual(expected_message, result.find("./Message").text)

    @mock.patch('scciclient.irmc.scci.get_firmware_upgrade_status')
    def test_failed_process_session_bios_status(self, mock_status):
        session_timeout = 180
        upgrade_type = 'bios'
        # Fake status from server
        status_fake = ET.Element(self)
        status_fake.append(ET.Element("Value", name="Value"))
        status_fake.append(ET.Element("Severity", name="Severity"))
        status_fake.append(ET.Element("Message", name="Message"))
        status_fake.find("./Value").text = '0'
        status_fake.find("./Severity").text = 'Error'
        status_fake.find("./Message").text = 'File not provided'

        mock_status.return_value = status_fake
        expected_status = 'Error'
        result = scci.process_session_status(self.irmc_info, session_timeout,
                                             upgrade_type)
        self.assertEqual(expected_status, result['upgrade_status'])

    @mock.patch('scciclient.irmc.scci.get_firmware_upgrade_status')
    def test_success_process_session_bios_status(self, mock_status):
        session_timeout = 180
        upgrade_type = 'bios'
        # Fake status from server
        status_fake = ET.Element(self)
        status_fake.append(ET.Element("Value", name="Value"))
        status_fake.append(ET.Element("Severity", name="Severity"))
        status_fake.append(ET.Element("Message", name="Message"))
        status_fake.find("./Value").text = '9'
        status_fake.find("./Severity").text = 'Information'
        status_fake.find("./Message").text = 'FLASH successful'

        mock_status.return_value = status_fake
        expected_status = 'Complete'
        result = scci.process_session_status(self.irmc_info, session_timeout,
                                             upgrade_type)
        self.assertEqual(expected_status, result['upgrade_status'])

    @mock.patch('scciclient.irmc.scci.get_firmware_upgrade_status')
    def test_failed_process_session_irmc_status(self, mock_status):
        session_timeout = 180
        upgrade_type = 'irmc'
        # Fake status from server
        status_fake = ET.Element(self)
        status_fake.append(ET.Element("Value", name="Value"))
        status_fake.append(ET.Element("Severity", name="Severity"))
        status_fake.append(ET.Element("Message", name="Message"))
        status_fake.find("./Value").text = '0'
        status_fake.find("./Severity").text = 'Error'
        status_fake.find("./Message").text = 'File not provided'

        mock_status.return_value = status_fake
        expected_status = 'Error'
        result = scci.process_session_status(self.irmc_info, session_timeout,
                                             upgrade_type)
        self.assertEqual(expected_status, result['upgrade_status'])

    @mock.patch('scciclient.irmc.scci.get_firmware_upgrade_status')
    def test_success_process_session_irmc_status(self, mock_status):
        session_timeout = 180
        upgrade_type = 'irmc'
        # Fake status from server
        status_fake = ET.Element(self)
        status_fake.append(ET.Element("Value", name="Value"))
        status_fake.append(ET.Element("Severity", name="Severity"))
        status_fake.append(ET.Element("Message", name="Message"))
        status_fake.find("./Value").text = '9'
        status_fake.find("./Severity").text = 'Information'
        status_fake.find("./Message").text = 'FLASH successful'

        mock_status.return_value = status_fake
        expected_status = 'Complete'
        result = scci.process_session_status(self.irmc_info, session_timeout,
                                             upgrade_type)
        self.assertEqual(expected_status, result['upgrade_status'])

    @mock.patch.object(builtins, 'open', autospec=True)
    def test_create_bios_firmware_upgrade(self, open_mock):
        upgrade_type = 'bios'
        self.requests_mock.post("http://" + self.irmc_address + "/biosupdate",
                                text="""<?xml version="1.0" encoding="UTF-8"?>
                                <Status>
                                    <Value>0</Value>
                                    <Severity>Information</Severity>
                                    <Message>No Error</Message>
                                </Status>""")
        bios_input = '/media/DATA/D3099-B1.UPC'
        open_mock.return_value = mock.mock_open(read_data="file").return_value
        client = scci.get_client(self.irmc_address,
                                 self.irmc_username,
                                 self.irmc_password,
                                 port=self.irmc_port,
                                 auth_method=self.irmc_auth_method,
                                 client_timeout=self.irmc_client_timeout,
                                 upgrade_type=upgrade_type)
        r = client(bios_input)
        self.assertEqual(r.status_code, 200)

    @mock.patch.object(builtins, 'open', side_effect=IOError, autospec=True)
    def test_create_fail_bios_firmware_upgrade(self, open_mock):
        upgrade_type = 'bios'
        self.requests_mock.post("http://" + self.irmc_address + "/biosupdate",
                                text="""<?xml version="1.0" encoding="UTF-8"?>
                                <Status>
                                <Value>6</Value>
                                <Severity>Error</Severity>
                                <Message>File not provided</Message>
                                </Status>
                                """)
        # Fake wrong file directory
        bios_input = '/media/DATA/D3099-B101.UPC'
        open_mock.return_value = mock.mock_open(read_data="file").return_value
        client = scci.get_client(self.irmc_address,
                                 self.irmc_username,
                                 self.irmc_password,
                                 port=self.irmc_port,
                                 auth_method=self.irmc_auth_method,
                                 client_timeout=self.irmc_client_timeout,
                                 upgrade_type=upgrade_type)

        self.assertRaises(scci.SCCIClientError, client, bios_input)

    @mock.patch.object(builtins, 'open', autospec=True)
    def test_create_irmc_firmware_upgrade(self, open_mock):
        upgrade_type = 'irmc'
        self.requests_mock.post("http://" + self.irmc_address +
                                "/irmcupdate?flashSelect=255",
                                text="""<?xml version="1.0" encoding="UTF-8"?>
                                <Status>
                                    <Value>0</Value>
                                    <Severity>Information</Severity>
                                    <Message>No Error</Message>
                                </Status>""")
        irmc_input = '/media/DATA/TX2540M1.bin'
        open_mock.return_value = mock.mock_open(read_data="file").return_value
        client = scci.get_client(self.irmc_address,
                                 self.irmc_username,
                                 self.irmc_password,
                                 port=self.irmc_port,
                                 auth_method=self.irmc_auth_method,
                                 client_timeout=self.irmc_client_timeout,
                                 upgrade_type=upgrade_type)
        r = client(irmc_input)
        self.assertEqual(r.status_code, 200)

    @mock.patch.object(builtins, 'open', side_effect=IOError, autospec=True)
    def test_create_fail_irmc_firmware_upgrade(self, open_mock):
        upgrade_type = 'irmc'
        self.requests_mock.post("http://" + self.irmc_address +
                                "/irmcupdate?flashSelect=255",
                                text="""<?xml version="1.0" encoding="UTF-8"?>
                                <Status>
                                <Value>6</Value>
                                <Severity>Error</Severity>
                                <Message>File not provided</Message>
                                </Status>
                                """)
        # Fake wrong file directory
        irmc_input = '/media/DATA/TX2540M1111.bin'
        mock_file_handle = mock.MagicMock(spec=io.BytesIO)
        open_mock.return_value = mock_file_handle
        client = scci.get_client(self.irmc_address,
                                 self.irmc_username,
                                 self.irmc_password,
                                 port=self.irmc_port,
                                 auth_method=self.irmc_auth_method,
                                 client_timeout=self.irmc_client_timeout,
                                 upgrade_type=upgrade_type)

        self.assertRaises(scci.SCCIClientError, client, irmc_input)



# Copyright 2017 FUJITSU LIMITED
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from pysnmp import error as snmp_error
from pysnmp import hlapi as snmp


BMC_NAME_OID = '1.3.6.1.4.1.231.2.10.2.2.10.3.4.1.3.1.1'
IRMC_FW_VERSION_OID = '1.3.6.1.4.1.231.2.10.2.2.10.3.4.1.4.1.1'
BIOS_FW_VERSION_OID = '1.3.6.1.4.1.231.2.10.2.2.10.4.1.1.11.1'
SERVER_MODEL_OID = '1.3.6.1.4.1.231.2.10.2.2.10.2.3.1.4.1'

SNMP_V1 = '1'
SNMP_V2C = '2c'
SNMP_V3 = '3'

SNMP_FAILURE_MSG = "SNMP operation '%s' failed: %s"


class SNMPFailure(Exception):
    """SNMP Failure

    This exception is used when invalid inputs are passed to
    the APIs exposed by this module.
    """
    def __init__(self, message):
        super(SNMPFailure, self).__init__(message)


class SNMPIRMCFirmwareFailure(SNMPFailure):
    """SNMP iRMC Firmware Failure

    This exception is used when error occurs when collecting iRMC firmware.
    """
    def __init__(self, message):
        super(SNMPIRMCFirmwareFailure, self).__init__(message)


class SNMPBIOSFirmwareFailure(SNMPFailure):
    """SNMP BIOS Firmware Failure

    This exception is used when error occurs when collecting BIOS firmware.
    """
    def __init__(self, message):
        super(SNMPBIOSFirmwareFailure, self).__init__(message)


class SNMPServerModelFailure(SNMPFailure):
    """SNMP Server Model Failure

    This exception is used when error occurs when collecting server model.
    """
    def __init__(self, message):
        super(SNMPServerModelFailure, self).__init__(message)


def get_irmc_firmware_version(snmp_client):
    """Get irmc firmware version of the node.

    :param snmp_client: an SNMP client object.
    :raises: SNMPFailure if SNMP operation failed.
    :returns: a string of bmc name and irmc firmware version.
    """

    try:
        bmc_name = snmp_client.get(BMC_NAME_OID)
        irmc_firm_ver = snmp_client.get(IRMC_FW_VERSION_OID)
        return ('%(bmc)s%(sep)s%(firm_ver)s' %
                {'bmc': bmc_name if bmc_name else '',
                 'firm_ver': irmc_firm_ver if irmc_firm_ver else '',
                 'sep': '-' if bmc_name and irmc_firm_ver else ''})
    except SNMPFailure as e:
        raise SNMPIRMCFirmwareFailure(
            SNMP_FAILURE_MSG % ("GET IRMC FIRMWARE VERSION", e))


def get_bios_firmware_version(snmp_client):
    """Get bios firmware version of the node.

    :param snmp_client: an SNMP client object.
    :raises: SNMPFailure if SNMP operation failed.
    :returns: a string of bios firmware version.
    """

    try:
        bios_firmware_version = snmp_client.get(BIOS_FW_VERSION_OID)
        return str(bios_firmware_version)
    except SNMPFailure as e:
        raise SNMPBIOSFirmwareFailure(
            SNMP_FAILURE_MSG % ("GET BIOS FIRMWARE VERSION", e))


def get_server_model(snmp_client):
    """Get server model of the node.

    :param snmp_client: an SNMP client object.
    :raises: SNMPFailure if SNMP operation failed.
    :returns: a string of server model.
    """

    try:
        server_model = snmp_client.get(SERVER_MODEL_OID)
        return str(server_model)
    except SNMPFailure as e:
        raise SNMPServerModelFailure(
            SNMP_FAILURE_MSG % ("GET SERVER MODEL", e))


class SNMPClient(object):
    """SNMP client object.

    Performs low level SNMP get and set operations. Encapsulates all
    interaction with PySNMP to simplify dynamic importing and unit testing.
    """

    def __init__(self, address, port, version,
                 read_community=None, write_community=None,
                 user=None, auth_proto=None, auth_key=None,
                 priv_proto=None, priv_key=None,
                 context_engine_id=None, context_name=None):
        self.address = address
        self.port = port
        self.version = version
        if self.version == SNMP_V3:
            self.user = user
            self.auth_proto = auth_proto
            self.auth_key = auth_key
            self.priv_proto = priv_proto
            self.priv_key = priv_key
        else:
            self.read_community = read_community
            self.write_community = write_community

        self.context_engine_id = context_engine_id
        self.context_name = context_name or ''
        self.snmp_engine = snmp.SnmpEngine()

    def _get_auth(self, write_mode=False):
        """Return the authorization data for an SNMP request.

        :param write_mode: `True` if write class SNMP command is
            executed. Default is `False`.
        :returns: Either
            :class:`pysnmp.hlapi.CommunityData`
            or :class:`pysnmp.hlapi.UsmUserData`
            object depending on SNMP version being used.
        """
        if self.version == SNMP_V3:
            # Handling auth/encryption credentials is not (yet) supported.
            # This version supports a security name analogous to community.
            return snmp.UsmUserData(self.user,
                                    authKey=self.auth_key,
                                    authProtocol=self.auth_proto,
                                    privKey=self.priv_key,
                                    privProtocol=self.priv_proto)
        else:
            mp_model = 1 if self.version == SNMP_V2C else 0
            return snmp.CommunityData(
                self.write_community if write_mode else self.read_community,
                mpModel=mp_model
            )

    def _get_transport(self):
        """Return the transport target for an SNMP request.

        :returns: A :class:
            `pysnmp.hlapi.UdpTransportTarget` object.
        :raises: :class:`pysnmp.error.PySnmpError` if the transport address
            is bad.
        """
        # The transport target accepts timeout and retries parameters, which
        # default to 1 (second) and 5 respectively. These are deemed sensible
        # enough to allow for an unreliable network or slow device.
        return snmp.UdpTransportTarget((self.address, self.port))

    def _get_context(self):
        """Return the SNMP context for an SNMP request.

        :returns: A :class:
            `pysnmp.hlapi.ContextData` object.
        :raises: :class:`pysnmp.error.PySnmpError` if SNMP context data
            is bad.
        """
        return snmp.ContextData(contextEngineId=self.context_engine_id,
                                contextName=self.context_name)

    def get(self, oid):
        """Use PySNMP to perform an SNMP GET operation on a single object.

        :param oid: The OID of the object to get.
        :raises: SNMPFailure if an SNMP request fails.
        :returns: The value of the requested object.
        """
        try:
            snmp_gen = snmp.getCmd(self.snmp_engine,
                                   self._get_auth(),
                                   self._get_transport(),
                                   self._get_context(),
                                   snmp.ObjectType(snmp.ObjectIdentity(oid)))
        except snmp_error.PySnmpError as e:
            raise SNMPFailure(SNMP_FAILURE_MSG % ("GET", e))

        error_indication, error_status, error_index, var_binds = next(snmp_gen)

        if error_indication:
            # SNMP engine-level error.
            raise SNMPFailure(SNMP_FAILURE_MSG % ("GET", error_indication))

        if error_status:
            # SNMP PDU error.
            raise SNMPFailure(
                "SNMP operation '%(operation)s' failed: %(error)s at"
                " %(index)s" %
                {'operation': "GET", 'error': error_status.prettyPrint(),
                 'index':
                     error_index and var_binds[int(error_index) - 1] or '?'})

        # We only expect a single value back
        name, val = var_binds[0]
        return val

    def get_next(self, oid):
        """Use PySNMP to perform an SNMP GET NEXT operation on a table object.

        :param oid: The OID of the object to get.
        :raises: SNMPFailure if an SNMP request fails.
        :returns: A list of values of the requested table object.
        """
        try:
            snmp_gen = snmp.nextCmd(self.snmp_engine,
                                    self._get_auth(),
                                    self._get_transport(),
                                    self._get_context(),
                                    snmp.ObjectType(snmp.ObjectIdentity(oid)),
                                    lexicographicMode=False)
        except snmp_error.PySnmpError as e:
            raise SNMPFailure(SNMP_FAILURE_MSG % ("GET_NEXT", e))

        vals = []
        for (error_indication, error_status, error_index,
                var_binds) in snmp_gen:
            if error_indication:
                # SNMP engine-level error.
                raise SNMPFailure(SNMP_FAILURE_MSG % ("GET_NEXT",
                                                      error_indication))

            if error_status:
                # SNMP PDU error.
                raise SNMPFailure(
                    "SNMP operation '%(operation)s' failed: %(error)s at"
                    " %(index)s" %
                    {'operation': "GET_NEXT",
                     'error': error_status.prettyPrint(),
                     'index':
                        error_index and var_binds[int(error_index) - 1]
                        or '?'})

            name, value = var_binds[0]
            vals.append(value)

        return vals

    def set(self, oid, value):
        """Use PySNMP to perform an SNMP SET operation on a single object.

        :param oid: The OID of the object to set.
        :param value: The value of the object to set.
        :raises: SNMPFailure if an SNMP request fails.
        """
        try:
            snmp_gen = snmp.setCmd(self.snmp_engine,
                                   self._get_auth(write_mode=True),
                                   self._get_transport(),
                                   self._get_context(),
                                   snmp.ObjectType(snmp.ObjectIdentity(oid),
                                                   value))
        except snmp_error.PySnmpError as e:
            raise SNMPFailure(SNMP_FAILURE_MSG % ("SET", e))

        error_indication, error_status, error_index, var_binds = next(snmp_gen)

        if error_indication:
            # SNMP engine-level error.
            raise SNMPFailure(SNMP_FAILURE_MSG % ("SET", error_indication))

        if error_status:
            # SNMP PDU error.
            raise SNMPFailure(
                "SNMP operation '%(operation)s' failed: %(error)s at"
                " %(index)s" %
                {'operation': "SET", 'error': error_status.prettyPrint(),
                 'index':
                     error_index and var_binds[int(error_index) - 1] or '?'})



# Copyright 2017 FUJITSU LIMITED
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import functools
import itertools

from pyghmi import exceptions as ipmi_exception
from pyghmi.ipmi import command as ipmi_command

# F1 1A - Get the number of specific PCI devices on baremetal
GET_PCI = '0x2E 0xF1 0x80 0x28 0x00 0x1A %s 0x00'

# F5 81 - GET TPM STATUS
GET_TPM_STATUS = '0x2E 0xF5 0x80 0x28 0x00 0x81 0xC0'


class IPMIFailure(Exception):
    """IPMI Failure

    This exception is used when IPMI operation failed.
    """
    def __init__(self, message):
        super(IPMIFailure, self).__init__(message)


class InvalidParameterValue(IPMIFailure):
    """Invalid Parameter Value Failure

    This exception is used when invalid parameter values are passed to
    the APIs exposed by this module.
    """
    def __init__(self, message):
        super(InvalidParameterValue, self).__init__(message)


def _parse_raw_bytes(raw_bytes):
    """Convert a string of hexadecimal values to decimal values parameters

    Example: '0x2E 0xF1 0x80 0x28 0x00 0x1A 0x01 0x00' is converted to:
              46, 241, [128, 40, 0, 26, 1, 0]

    :param raw_bytes: string of hexadecimal values
    :returns: 3 decimal values
    """
    bytes_list = [int(x, base=16) for x in raw_bytes.split()]
    return bytes_list[0], bytes_list[1], bytes_list[2:]


def _send_raw_command(ipmicmd, raw_bytes):
    """Use IPMI command object to send raw ipmi command to BMC

    :param ipmicmd: IPMI command object
    :param raw_bytes: string of hexadecimal values. This is commonly used
        for certain vendor specific commands.
    :returns: dict -- The response from IPMI device
    """

    netfn, command, data = _parse_raw_bytes(raw_bytes)
    response = ipmicmd.raw_command(netfn, command, data=data)

    return response


def get_tpm_status(d_info):
    """Get the TPM support status.

    Get the TPM support status of the node.

    :param d_info: the list of ipmitool parameters for accessing a node.
    :returns: TPM support status
    """

    # note:
    # Get TPM support status : ipmi cmd '0xF5', valid flags '0xC0'
    #
    # $ ipmitool raw 0x2E 0xF5 0x80 0x28 0x00 0x81 0xC0
    #
    # Raw response:
    # 80 28 00 C0 C0: True
    # 80 28 00 -- --: False (other values than 'C0 C0')

    ipmicmd = ipmi_command.Command(
        bmc=d_info['irmc_address'],
        userid=d_info['irmc_username'].encode('utf-8'),
        password=d_info['irmc_password'].encode('utf-8'))
    try:
        response = _send_raw_command(ipmicmd, GET_TPM_STATUS)
        if response['code'] != 0:
            raise IPMIFailure(
                "IPMI operation '%(operation)s' failed: %(error)s" %
                {'operation': "GET TMP status",
                 'error': response.get('error')})
        out = ' '.join('{:02X}'.format(x) for x in response['data'])
        return out is not None and out[-5:] == 'C0 C0'

    except ipmi_exception.IpmiException as e:
        raise IPMIFailure(
            "IPMI operation '%(operation)s' failed: %(error)s" %
            {'operation': "GET TMP status", 'error': e})


def _pci_seq(ipmicmd):
    """Get output of ipmiraw command and the ordinal numbers.

    :param ipmicmd: IPMI command object.
    :returns: List of tuple contain ordinal number and output of ipmiraw
    command.
    """
    for i in range(1, 0xff + 1):
        try:
            res = _send_raw_command(ipmicmd, GET_PCI % hex(i))
            yield i, res
        except ipmi_exception.IpmiException as e:
            raise IPMIFailure(
                "IPMI operation '%(operation)s' failed: %(error)s" %
                {'operation': "GET PCI device quantity", 'error': e})


def get_pci_device(d_info, pci_device_ids):
    """Get quantity of PCI devices.

    Get quantity of PCI devices of the node.

    :param d_info: the list of ipmitool parameters for accessing a node.
    :param pci_device_ids: the list contains pairs of <vendorID>/<deviceID> for
    PCI devices.
    :returns: the number of PCI devices.
    """

    # note:
    # Get quantity of PCI devices:
    # ipmi cmd '0xF1'
    #
    # $ ipmitool raw 0x2E 0xF1 0x80 0x28 0x00 0x1A 0x01 0x00
    #
    # Raw response:
    # 80 28 00 00 00 05 data1 data2 34 17 76 11 00 04
    # 01

    # data1: 2 octet of VendorID
    # data2: 2 octet of DeviceID

    ipmicmd = ipmi_command.Command(
        bmc=d_info['irmc_address'],
        userid=d_info['irmc_username'].encode('utf-8'),
        password=d_info['irmc_password'].encode('utf-8'))

    response = itertools.takewhile(
        lambda y: (y[1]['code'] != 0xC9 and y[1].get('error') is None),
        _pci_seq(ipmicmd))

    def _pci_count(accm, v):
        out = v[1]['data']
        # if system returns value, record id will be increased.
        pci_id = "0x{:02x}{:02x}/0x{:02x}{:02x}".format(
            out[7], out[6], out[9], out[8])
        return accm + 1 if pci_id in pci_device_ids else accm

    device_count = functools.reduce(_pci_count, response, 0)

    return device_count



# Copyright 2017 FUJITSU LIMITED
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
Test class for iRMC eLCM functionality.
"""

import time
from unittest import mock

import testtools

from scciclient.irmc import elcm
from scciclient.irmc import scci
from scciclient.irmc.viom import elcm as viom_elcm


class ELCMVIOMClientTestCase(testtools.TestCase):
    """Test for ELCMViomClient."""

    def setUp(self):
        super(ELCMVIOMClientTestCase, self).setUp()

        self.irmc_info = {
            'irmc_address': '10.124.196.159',
            'irmc_username': 'admin',
            'irmc_password': 'admin0',
            'irmc_port': 80,
            'irmc_auth_method': 'basic',
            'irmc_client_timeout': 60,
        }
        self.client = viom_elcm.ELCMVIOMClient(self.irmc_info)
        self.session_id = '10'

    @staticmethod
    def _session_status_resp(session_id, status):
        return {'Session': {'Id': session_id,
                            'Status': status}}

    @staticmethod
    def _session_log_resp(session_id):
        return {'Sessionlog':
                {'Id': session_id,
                 'Entries':
                    {'Entry':
                        {'@date': '2017/04/24 18:06:27',
                         '#text': 'CreateSession: create'}}}}

    @mock.patch.object(elcm, 'elcm_session_get_status')
    @mock.patch.object(time, 'sleep')
    def test__wait_session(self, mock_sleep, mock_get_session):
        mock_get_session.side_effect = [
            self._session_status_resp(self.session_id, 'running'),
            self._session_status_resp(self.session_id, 'activated'),
            self._session_status_resp(self.session_id, 'terminated regularly')]

        self.assertEqual({}, self.client._wait_session(self.session_id))
        mock_get_session.assert_has_calls(
            [mock.call(self.irmc_info, self.session_id) for i in range(0, 3)])
        self.assertEqual(3, mock_get_session.call_count)
        self.assertEqual(2, mock_sleep.call_count)

    @mock.patch.object(elcm, 'elcm_session_get_status')
    @mock.patch.object(elcm, 'elcm_session_get_log')
    def test__wait_session_error(self, mock_get_log, mock_get_session):
        mock_get_session.return_value = (
            self._session_status_resp(self.session_id,
                                      'terminated with error'))
        mock_get_log.return_value = self._session_log_resp(self.session_id)

        self.assertRaises(scci.SCCIClientError,
                          self.client._wait_session,
                          self.session_id)
        mock_get_session.assert_called_once_with(self.irmc_info,
                                                 self.session_id)
        mock_get_log.assert_called_once_with(irmc_info=self.irmc_info,
                                             session_id=self.session_id)

    @mock.patch.object(elcm, 'elcm_session_get_status')
    @mock.patch.object(elcm, 'elcm_session_get_log')
    def test__wait_session_error_log_error(self, mock_get_log,
                                           mock_get_session):
        mock_get_session.return_value = (
            self._session_status_resp(self.session_id,
                                      'terminated with error'))
        mock_get_log.side_effect = scci.SCCIClientError(
            'got an error')

        self.assertRaises(scci.SCCIClientError,
                          self.client._wait_session,
                          self.session_id)
        mock_get_session.assert_called_once_with(self.irmc_info,
                                                 self.session_id)
        mock_get_log.assert_called_once_with(irmc_info=self.irmc_info,
                                             session_id=self.session_id)

    @mock.patch.object(elcm, 'elcm_session_get_status')
    @mock.patch.object(elcm, 'elcm_session_get_log')
    @mock.patch.object(time, 'time')
    @mock.patch.object(time, 'sleep')
    def test__wait_session_timeout(self, mock_sleep, mock_time, mock_get_log,
                                   mock_get_session):
        timeout = 1800
        mock_time.side_effect = [100, 101 + timeout]
        mock_get_session.return_value = (
            self._session_status_resp(self.session_id, 'running'))
        mock_get_log.return_value = self._session_log_resp(self.session_id)

        self.assertRaises(elcm.ELCMSessionTimeout,
                          self.client._wait_session,
                          self.session_id)
        mock_get_session.assert_called_once_with(self.irmc_info,
                                                 self.session_id)
        mock_get_log.assert_called_once_with(irmc_info=self.irmc_info,
                                             session_id=self.session_id)

    @mock.patch.object(elcm, 'elcm_session_get_status')
    @mock.patch.object(elcm, 'elcm_session_get_log')
    @mock.patch.object(time, 'time')
    @mock.patch.object(time, 'sleep')
    def test__wait_session_timeout_specified(self, mock_sleep, mock_time,
                                             mock_get_log, mock_get_session):
        timeout = 60
        mock_time.side_effect = [100, 101 + timeout]
        mock_get_session.return_value = (
            self._session_status_resp(self.session_id, 'running'))
        mock_get_log.return_value = self._session_log_resp(self.session_id)

        self.assertRaises(elcm.ELCMSessionTimeout,
                          self.client._wait_session,
                          self.session_id,
                          timeout=timeout)
        mock_get_session.assert_called_once_with(self.irmc_info,
                                                 self.session_id)
        mock_get_log.assert_called_once_with(irmc_info=self.irmc_info,
                                             session_id=self.session_id)

    @mock.patch.object(elcm, 'elcm_session_get_status')
    @mock.patch.object(elcm, 'elcm_session_get_log')
    @mock.patch.object(time, 'time')
    @mock.patch.object(time, 'sleep')
    def test__wait_session_timeout_log_error(self, mock_sleep, mock_time,
                                             mock_get_log, mock_get_session):
        timeout = 1800
        mock_time.side_effect = [100, 101 + timeout]
        mock_get_session.return_value = (
            self._session_status_resp(self.session_id, 'running'))
        mock_get_log.side_effect = scci.SCCIClientError('got an error')

        self.assertRaises(elcm.ELCMSessionTimeout,
                          self.client._wait_session,
                          self.session_id)
        mock_get_session.assert_called_once_with(self.irmc_info,
                                                 self.session_id)
        mock_get_log.assert_called_once_with(irmc_info=self.irmc_info,
                                             session_id=self.session_id)

    @mock.patch.object(elcm, 'elcm_profile_set')
    @mock.patch.object(viom_elcm.ELCMVIOMClient, '_wait_session',
                       return_value={})
    def test_set_profile(self, mock_wait, mock_set):
        adapter_config = {
            'ViomManage': {'Manage': True},
            'InitBoot': True}
        mock_set.return_value = self._session_status_resp(self.session_id,
                                                          'activated')
        self.assertIsNone(self.client.set_profile(adapter_config))
        mock_wait.assert_called_once_with(self.session_id)
        mock_set.assert_called_once_with(
            self.irmc_info,
            {'Server':
                {'AdapterConfigIrmc':
                    {'ViomManage': {'Manage': True},
                     'InitBoot': True,
                     '@Processing': 'execute'},
                 '@Version': '1.01'}})

    @mock.patch.object(elcm, 'elcm_profile_create')
    @mock.patch.object(viom_elcm.ELCMVIOMClient, '_wait_session',
                       return_value={})
    @mock.patch.object(elcm, 'elcm_profile_get')
    def _test_get_profile(self, mock_get, mock_wait, mock_create):
        mock_create.return_value = self._session_status_resp(self.session_id,
                                                             'activated')
        adapter_data = {
            'Server': {
                'AdapterConfigIrmc':
                    {'ViomManage': {'Manage': True},
                     'InitBoot': True}}}
        mock_get.return_value = adapter_data
        self.assertEqual(adapter_data, self.client.get_profile())
        mock_create.assert_called_once_with(self.irmc_info,
                                            'Server/AdapterConfigIrmc')
        mock_wait.assert_called_once_with(self.session_id)
        mock_get.assert_called_once_with(self.irmc_info,
                                         'AdapterConfigIrmc')

    @mock.patch.object(elcm, 'elcm_profile_delete')
    def test_get_profile_with_delete(self, mock_delete):
        self._test_get_profile()
        mock_delete.assert_called_once_with(self.irmc_info,
                                            'AdapterConfigIrmc')

    @mock.patch.object(elcm, 'elcm_profile_delete')
    def test_get_profile_without_delete(self, mock_delete):
        mock_delete.side_effect = elcm.ELCMProfileNotFound('not found')
        self._test_get_profile()
        mock_delete.assert_called_once_with(self.irmc_info,
                                            'AdapterConfigIrmc')


class VIOMTableTestCase(testtools.TestCase):
    """Test for VIOM table."""

    def setUp(self):
        super(VIOMTableTestCase, self).setUp()

    @staticmethod
    def _sample_manage_table():
        return viom_elcm.ManageTable(
            manage=True,
            identification='identity',
            force=True)

    @staticmethod
    def _sample_manage_table_json():
        return {
            'Manage': True,
            'Identification': 'identity',
            'Force': True,
        }

    @staticmethod
    def _add_sample_cards_to_slot(slot):
        slot.add_card(VIOMTableTestCase._sample_onboard_card())
        slot.add_card(VIOMTableTestCase._sample_addon_card())

    @staticmethod
    def _sample_slot_json():
        return {
            '@SlotIdx': 0,
            'OnboardControllers': {
                'OnboardController': [
                    VIOMTableTestCase._sample_onboard_card_json()
                ]
            },
            'AddOnCards': {
                'AddOnCard': [
                    VIOMTableTestCase._sample_addon_card_json()
                ]
            }
        }

    @staticmethod
    def _sample_onboard_card():
        onboard_card = viom_elcm.OnboardCard(1, viom_elcm.LANAdapter())
        onboard_card.add_port(VIOMTableTestCase._sample_lan_port(1))
        return onboard_card

    @staticmethod
    def _sample_onboard_card_json():
        return {
            '@OnboardControllerIdx': 1,
            'LANAdapter': {
                'Ports': {
                    'Port': [
                        VIOMTableTestCase._sample_lan_port_json(1),
                    ]
                }
            }
        }

    @staticmethod
    def _sample_addon_card():
        addon_card = viom_elcm.AddOnCard(2, viom_elcm.FCAdapter())
        addon_card.add_port(VIOMTableTestCase._sample_fc_port(1))
        return addon_card

    @staticmethod
    def _sample_addon_card_json():
        return {
            '@AddOnCardIdx': 2,
            'FCAdapter': {
                'Ports': {
                    'Port': [
                        VIOMTableTestCase._sample_fc_port_json(1),
                    ]
                }
            }
        }

    @staticmethod
    def _sample_lan_port(port_id=1):
        return viom_elcm.LANPort(port_id)

    @staticmethod
    def _sample_lan_port_json(port_id=1):
        return {
            '@PortIdx': port_id,
            'PortEnable': True,
            'BootProtocol': 'None',
            'BootPriority': 1,
        }

    @staticmethod
    def _sample_fc_port(port_id=1):
        return viom_elcm.FCPort(
            port_id, boot=VIOMTableTestCase._sample_fc_boot())

    @staticmethod
    def _sample_fc_port_json(port_id=1):
        sample = {
            '@PortIdx': port_id,
            'PortEnable': True,
            'BootProtocol': 'FC',
            'BootPriority': 1,
        }
        sample.update(
            VIOMTableTestCase._sample_fc_boot_json())
        return sample

    @staticmethod
    def _sample_cna_port(port_id=1):
        cna_port = viom_elcm.CNAPort(port_id)
        cna_port.add_function(
            VIOMTableTestCase._sample_lan_function())
        cna_port.add_function(
            VIOMTableTestCase._sample_iscsi_function())
        return cna_port

    @staticmethod
    def _sample_cna_port_json(port_id=1):
        return {
            '@PortIdx': port_id,
            'PortEnable': True,
            'Functions': {
                'Function': [
                    VIOMTableTestCase._sample_lan_function_json(),
                    VIOMTableTestCase._sample_iscsi_function_json()
                ]
            }
        }

    @staticmethod
    def _sample_lan_function():
        return viom_elcm.LANFunction(1, function_enable=False)

    @staticmethod
    def _sample_lan_function_json():
        return {
            '@FunctionIdx': 1,
            'LANFunction': {
                'FunctionEnable': False,
                'BootProtocol': 'None',
                'BootPriority': 1,
            }
        }

    @staticmethod
    def _sample_fcoe_function():
        return viom_elcm.FCoEFunction(
            2, boot=VIOMTableTestCase._sample_fc_boot())

    @staticmethod
    def _sample_fcoe_function_json():
        sample = {
            '@FunctionIdx': 2,
            'FCoEFunction': {
                'FunctionEnable': True,
                'BootProtocol': 'FC',
                'BootPriority': 1,
            },
        }
        sample['FCoEFunction'].update(
            VIOMTableTestCase._sample_fc_boot_json())
        return sample

    @staticmethod
    def _sample_iscsi_function():
        return viom_elcm.ISCSIFunction(
            3, boot=VIOMTableTestCase._sample_iscsi_boot())

    @staticmethod
    def _sample_iscsi_function_json():
        sample = {
            '@FunctionIdx': 3,
            'ISCSIFunction': {
                'FunctionEnable': True,
                'BootProtocol': 'ISCSI',
                'BootPriority': 1,
            },
        }
        sample['ISCSIFunction'].update(
            VIOMTableTestCase._sample_iscsi_boot_json())
        return sample

    @staticmethod
    def _sample_fc_boot(boot_prio=None):
        if boot_prio:
            fc_boot = viom_elcm.FCBoot(boot_prio=boot_prio)
        else:
            fc_boot = viom_elcm.FCBoot()
        fc_boot.add_target(VIOMTableTestCase._sample_fc_target())
        return fc_boot

    @staticmethod
    def _sample_fc_boot_json():
        return {
            'FCBootEnvironment': {
                'FCTargets': {
                    'FCTarget': [VIOMTableTestCase._sample_fc_target_json()]
                },
                'FCLinkSpeed': 'auto',
                'FCTopology': 'auto_loop',
            }
        }

    @staticmethod
    def _sample_fc_target(wwpn='11:22:33:44:55'):
        return viom_elcm.FCTarget(wwpn)

    @staticmethod
    def _sample_fc_target_json(target_idx=1, wwpn='11:22:33:44:55'):
        return {'@FCTargetIdx': target_idx,
                'TargetWWPN': wwpn,
                'TargetLUN': 0}

    @staticmethod
    def _sample_iscsi_boot(boot_prio=None):
        if boot_prio:
            return viom_elcm.ISCSIBoot(
                VIOMTableTestCase._sample_iscsi_initiator(),
                VIOMTableTestCase._sample_iscsi_target(),
                boot_prio=boot_prio)
        else:
            return viom_elcm.ISCSIBoot(
                VIOMTableTestCase._sample_iscsi_initiator(),
                VIOMTableTestCase._sample_iscsi_target())

    @staticmethod
    def _sample_iscsi_boot_json():
        return {
            'ISCSIBootEnvironment': {
                'ISCSIInitiator':
                    VIOMTableTestCase._sample_iscsi_initiator_json(),
                'ISCSITarget':
                    VIOMTableTestCase._sample_iscsi_target_json()
            }
        }

    @staticmethod
    def _sample_iscsi_initiator():
        return viom_elcm.ISCSIInitiator(
            iqn='iqn-2017-04.com.fujitsu:001',
            ip='192.168.1.11',
            subnet='255.255.255.0',
            gateway='192.168.1.1')

    @staticmethod
    def _sample_iscsi_initiator_json():
        return {
            'DHCPUsage': False,
            'Name': 'iqn-2017-04.com.fujitsu:001',
            'IPv4Address': '192.168.1.11',
            'SubnetMask': '255.255.255.0',
            'GatewayIPv4Address': '192.168.1.1',
            'VLANId': 0,
        }

    @staticmethod
    def _sample_iscsi_target():
        return viom_elcm.ISCSITarget(
            iqn='iqn-2017-04.com.fujitsu:101',
            ip='192.168.2.22',
            auth_method='CHAP',
            chap_user='chap_user',
            chap_secret='chap_secret')

    @staticmethod
    def _sample_iscsi_target_json():
        return {
            'DHCPUsage': False,
            'Name': 'iqn-2017-04.com.fujitsu:101',
            'IPv4Address': '192.168.2.22',
            'PortNumber': 3260,
            'BootLUN': 0,
            'AuthenticationMethod': 'CHAP',
            'ChapUserName': 'chap_user',
            'ChapSecret': 'chap_secret',
        }

    def test_root(self):
        root = viom_elcm.VIOMTable(
            viom_boot_enable=True,
            init_boot=True,
            processing='execute',
            mode='new')
        root.set_manage_table(VIOMTableTestCase._sample_manage_table())
        self.assertIsNone(root.get_slot(0, create=False))
        slot = root.get_slot(0)
        VIOMTableTestCase._add_sample_cards_to_slot(slot)
        self.assertEqual(slot, root.get_slot(0))
        expected_json = {
            'VIOMManage': VIOMTableTestCase._sample_manage_table_json(),
            'InitBoot': True,
            'VIOMBootEnable': True,
            '@Processing': 'execute',
            'Mode': 'new',
            'Slots': {
                'Slot': [
                    VIOMTableTestCase._sample_slot_json()
                ]
            },
            '@Version': '1.00',
        }
        self.assertEqual(expected_json, root.get_json())

    def test_root_empty(self):
        root = viom_elcm.VIOMTable()
        self.assertEqual({'@Version': '1.00'}, root.get_json())

    def test_root_detail(self):
        root = viom_elcm.VIOMTable(
            use_virtual_addresses=True,
            viom_boot_enable=True,
            boot_menu_enable=False,
            sriov=False,
            smux='None',
            init_boot=True,
            processing='execute',
            mode='modify')
        root.set_manage_table(VIOMTableTestCase._sample_manage_table())
        VIOMTableTestCase._add_sample_cards_to_slot(root.get_slot(0))
        expected_json = {
            'VIOMManage': VIOMTableTestCase._sample_manage_table_json(),
            'UseVirtualAddresses': True,
            'VIOMBootEnable': True,
            'BootMenuEnable': False,
            'SRIOV': False,
            'Smux': 'None',
            'InitBoot': True,
            '@Processing': 'execute',
            'Mode': 'modify',
            'Slots': {
                'Slot': [
                    VIOMTableTestCase._sample_slot_json()
                ]
            },
            '@Version': '1.00',
        }
        self.assertEqual(expected_json, root.get_json())

    def test_manage_table(self):
        manage_table = viom_elcm.ManageTable(
            manage=True,
            identification='identity',
            force=True)
        expected_json = {
            'Manage': True,
            'Identification': 'identity',
            'Force': True,
        }
        self.assertEqual(expected_json, manage_table.get_json())

    def test_manage_table_detail(self):
        manage_table = viom_elcm.ManageTable(
            manage=True,
            identification='identity',
            force=False,
            trap_destination='192.168.3.33',
            preferred_version='2.6')
        expected_json = {
            'Manage': True,
            'Identification': 'identity',
            'Force': False,
            'TrapDestination': '192.168.3.33',
            'PreferredInventoryVersion': '2.6',
        }
        self.assertEqual(expected_json, manage_table.get_json())

    def test_slot(self):
        slot = viom_elcm.Slot(0)
        onboard_card = VIOMTableTestCase._sample_onboard_card()
        slot.add_card(onboard_card)
        addon_card = VIOMTableTestCase._sample_addon_card()
        slot.add_card(addon_card)
        expected_json = {
            '@SlotIdx': 0,
            'OnboardControllers': {
                'OnboardController': [
                    VIOMTableTestCase._sample_onboard_card_json()
                ]
            },
            'AddOnCards': {
                'AddOnCard': [
                    VIOMTableTestCase._sample_addon_card_json()
                ]
            }
        }
        self.assertEqual(expected_json, slot.get_json())
        self.assertEqual(onboard_card, slot.get_onboard_card(1))
        self.assertEqual(addon_card, slot.get_addon_card(2))

    def test_slot_only_onboard(self):
        slot = viom_elcm.Slot(1)
        card = VIOMTableTestCase._sample_onboard_card()
        slot.add_card(card)
        expected_json = {
            '@SlotIdx': 1,
            'OnboardControllers': {
                'OnboardController': [
                    VIOMTableTestCase._sample_onboard_card_json()
                ]
            },
        }
        self.assertEqual(expected_json, slot.get_json())
        self.assertEqual(card, slot.get_onboard_card(1))

    def test_slot_only_addon(self):
        slot = viom_elcm.Slot(2)
        card = VIOMTableTestCase._sample_addon_card()
        slot.add_card(card)
        expected_json = {
            '@SlotIdx': 2,
            'AddOnCards': {
                'AddOnCard': [
                    VIOMTableTestCase._sample_addon_card_json()
                ]
            }
        }
        self.assertEqual(expected_json, slot.get_json())
        self.assertEqual(card, slot.get_addon_card(2))

    def test_onboard_card(self):
        onboard_card = viom_elcm.OnboardCard(1, viom_elcm.LANAdapter())
        port1 = VIOMTableTestCase._sample_lan_port(1)
        port2 = VIOMTableTestCase._sample_lan_port(2)
        onboard_card.add_port(port1)
        onboard_card.add_port(port2)
        expected_json = {
            '@OnboardControllerIdx': 1,
            'LANAdapter': {
                'Ports': {
                    'Port': [
                        VIOMTableTestCase._sample_lan_port_json(1),
                        VIOMTableTestCase._sample_lan_port_json(2)
                    ]
                }
            }
        }
        self.assertEqual(expected_json, onboard_card.get_json())
        self.assertEqual(port1, onboard_card.get_port(1))
        self.assertEqual(port2, onboard_card.get_port(2))

    def test_addon_card(self):
        addon_card = viom_elcm.AddOnCard(2, viom_elcm.FCAdapter())
        port1 = VIOMTableTestCase._sample_fc_port(1)
        port2 = VIOMTableTestCase._sample_fc_port(2)
        addon_card.add_port(port1)
        addon_card.add_port(port2)
        expected_json = {
            '@AddOnCardIdx': 2,
            'FCAdapter': {
                'Ports': {
                    'Port': [
                        VIOMTableTestCase._sample_fc_port_json(1),
                        VIOMTableTestCase._sample_fc_port_json(2)
                    ]
                }
            }
        }
        self.assertEqual(expected_json, addon_card.get_json())
        self.assertEqual(port1, addon_card.get_port(1))
        self.assertEqual(port2, addon_card.get_port(2))

    def test_lan_adapter(self):
        lan_adapter = viom_elcm.LANAdapter()
        port1 = VIOMTableTestCase._sample_lan_port(1)
        port2 = VIOMTableTestCase._sample_lan_port(2)
        lan_adapter.add_port(port1)
        lan_adapter.add_port(port2)
        expected_json = {
            'LANAdapter': {
                'Ports': {
                    'Port': [
                        VIOMTableTestCase._sample_lan_port_json(1),
                        VIOMTableTestCase._sample_lan_port_json(2)
                    ]
                }
            }
        }
        self.assertEqual(expected_json, lan_adapter.get_json())
        self.assertEqual(port1, lan_adapter.get_port(1))
        self.assertEqual(port2, lan_adapter.get_port(2))

    def test_fc_adapter(self):
        fc_adapter = viom_elcm.FCAdapter()
        port1 = VIOMTableTestCase._sample_fc_port(1)
        port2 = VIOMTableTestCase._sample_fc_port(2)
        fc_adapter.add_port(port1)
        fc_adapter.add_port(port2)
        expected_json = {
            'FCAdapter': {
                'Ports': {
                    'Port': [
                        VIOMTableTestCase._sample_fc_port_json(1),
                        VIOMTableTestCase._sample_fc_port_json(2)
                    ]
                }
            }
        }
        self.assertEqual(expected_json, fc_adapter.get_json())
        self.assertEqual(port1, fc_adapter.get_port(1))
        self.assertEqual(port2, fc_adapter.get_port(2))

    def test_cna_adapter(self):
        cna_adapter = viom_elcm.CNAAdapter()
        port1 = VIOMTableTestCase._sample_cna_port(1)
        port2 = VIOMTableTestCase._sample_cna_port(2)
        cna_adapter.add_port(port1)
        cna_adapter.add_port(port2)
        expected_json = {
            'CNAAdapter': {
                'Ports': {
                    'Port': [
                        VIOMTableTestCase._sample_cna_port_json(1),
                        VIOMTableTestCase._sample_cna_port_json(2)
                    ]
                }
            }
        }
        self.assertEqual(expected_json, cna_adapter.get_json())
        self.assertEqual(port1, cna_adapter.get_port(1))
        self.assertEqual(port2, cna_adapter.get_port(2))

    def test_lan_port(self):
        lan_port = viom_elcm.LANPort(1)
        expected_json = {
            '@PortIdx': 1,
            'PortEnable': True,
            'BootProtocol': 'None',
            'BootPriority': 1,
        }
        self.assertEqual(expected_json, lan_port.get_json())

    def test_lan_port_pxe_boot(self):
        lan_port = viom_elcm.LANPort(
            2, boot=viom_elcm.PXEBoot(boot_prio=3))
        expected_json = {
            '@PortIdx': 2,
            'PortEnable': True,
            'BootProtocol': 'PXE',
            'BootPriority': 3,
        }
        self.assertEqual(expected_json, lan_port.get_json())

    def test_lan_port_virtualize_mac(self):
        lan_port = viom_elcm.LANPort(
            3, use_virtual_addresses=True, mac='11:22:33:44:55')
        expected_json = {
            '@PortIdx': 3,
            'PortEnable': True,
            'BootProtocol': 'None',
            'BootPriority': 1,
            'UseVirtualAddresses': True,
            'VirtualAddress': {
                'MAC': '11:22:33:44:55'
            }
        }
        self.assertEqual(expected_json, lan_port.get_json())

    def test_lan_port_detail(self):
        lan_port = viom_elcm.LANPort(4, port_enable=False, sriov=True)
        expected_json = {
            '@PortIdx': 4,
            'PortEnable': False,
            'BootProtocol': 'None',
            'BootPriority': 1,
            'SRIOV': True,
        }
        self.assertEqual(expected_json, lan_port.get_json())

    def test_fc_port(self):
        fc_port = viom_elcm.FCPort(
            1, boot=VIOMTableTestCase._sample_fc_boot())
        expected_json = {
            '@PortIdx': 1,
            'PortEnable': True,
            'BootProtocol': 'FC',
            'BootPriority': 1,
        }
        expected_json.update(
            VIOMTableTestCase._sample_fc_boot_json())
        self.assertEqual(expected_json, fc_port.get_json())

    def test_fc_port_boot_priority(self):
        fc_port = viom_elcm.FCPort(
            2, boot=VIOMTableTestCase._sample_fc_boot(boot_prio=3))
        expected_json = {
            '@PortIdx': 2,
            'PortEnable': True,
            'BootProtocol': 'FC',
            'BootPriority': 3,
        }
        expected_json.update(
            VIOMTableTestCase._sample_fc_boot_json())
        self.assertEqual(expected_json, fc_port.get_json())

    def test_fc_port_virtualize_wwn(self):
        fc_port = viom_elcm.FCPort(
            3, boot=VIOMTableTestCase._sample_fc_boot(),
            use_virtual_addresses=True,
            wwnn='11:22:33:44:55', wwpn='66:77:88:99:00')
        expected_json = {
            '@PortIdx': 3,
            'PortEnable': True,
            'BootProtocol': 'FC',
            'BootPriority': 1,
            'UseVirtualAddresses': True,
            'VirtualAddress': {
                'WWNN': '11:22:33:44:55',
                'WWPN': '66:77:88:99:00'
            },
        }
        expected_json.update(
            VIOMTableTestCase._sample_fc_boot_json())
        self.assertEqual(expected_json, fc_port.get_json())

    def test_fc_port_detail(self):
        fc_port = viom_elcm.FCPort(
            4, boot=VIOMTableTestCase._sample_fc_boot(),
            port_enable=False, sriov=True)
        expected_json = {
            '@PortIdx': 4,
            'PortEnable': False,
            'BootProtocol': 'FC',
            'BootPriority': 1,
            'SRIOV': True,
        }
        expected_json.update(
            VIOMTableTestCase._sample_fc_boot_json())
        self.assertEqual(expected_json, fc_port.get_json())

    def test_cna_port_fcoe(self):
        cna_port = viom_elcm.CNAPort(1)
        lan_function = VIOMTableTestCase._sample_lan_function()
        fcoe_function = VIOMTableTestCase._sample_fcoe_function()
        cna_port.add_function(lan_function)
        cna_port.add_function(fcoe_function)
        expected_json = {
            '@PortIdx': 1,
            'PortEnable': True,
            'Functions': {
                'Function': [
                    VIOMTableTestCase._sample_lan_function_json(),
                    VIOMTableTestCase._sample_fcoe_function_json()
                ]
            }
        }
        self.assertEqual(expected_json, cna_port.get_json())
        self.assertEqual(lan_function, cna_port.get_function(1))
        self.assertEqual(fcoe_function, cna_port.get_function(2))

    def test_cna_port_iscsi(self):
        cna_port = viom_elcm.CNAPort(2)
        cna_port.add_function(
            VIOMTableTestCase._sample_lan_function())
        cna_port.add_function(
            VIOMTableTestCase._sample_iscsi_function())
        expected_json = {
            '@PortIdx': 2,
            'PortEnable': True,
            'Functions': {
                'Function': [
                    VIOMTableTestCase._sample_lan_function_json(),
                    VIOMTableTestCase._sample_iscsi_function_json()
                ]
            }
        }
        self.assertEqual(expected_json, cna_port.get_json())

    def test_cna_port_disable(self):
        cna_port = viom_elcm.CNAPort(3, port_enable=False)
        cna_port.add_function(
            VIOMTableTestCase._sample_lan_function())
        cna_port.add_function(
            VIOMTableTestCase._sample_iscsi_function())
        expected_json = {
            '@PortIdx': 3,
            'PortEnable': False,
            'Functions': {
                'Function': [
                    VIOMTableTestCase._sample_lan_function_json(),
                    VIOMTableTestCase._sample_iscsi_function_json()
                ]
            }
        }
        self.assertEqual(expected_json, cna_port.get_json())

    def test_lan_function(self):
        lan_function = viom_elcm.LANFunction(1)
        expected_json = {
            '@FunctionIdx': 1,
            'LANFunction': {
                'FunctionEnable': True,
                'BootProtocol': 'None',
                'BootPriority': 1,
            }
        }
        self.assertEqual(expected_json, lan_function.get_json())

    def test_lan_function_pxe_boot(self):
        lan_function = viom_elcm.LANFunction(
            2, boot=viom_elcm.PXEBoot(boot_prio=3))
        expected_json = {
            '@FunctionIdx': 2,
            'LANFunction': {
                'FunctionEnable': True,
                'BootProtocol': 'PXE',
                'BootPriority': 3,
            }
        }
        self.assertEqual(expected_json, lan_function.get_json())

    def test_lan_function_virtualize_mac(self):
        lan_function = viom_elcm.LANFunction(
            3, use_virtual_addresses=True, mac='11:22:33:44:55')
        expected_json = {
            '@FunctionIdx': 3,
            'LANFunction': {
                'FunctionEnable': True,
                'BootProtocol': 'None',
                'BootPriority': 1,
                'UseVirtualAddresses': True,
                'VirtualAddress': {
                    'MAC': '11:22:33:44:55'
                },
            }
        }
        self.assertEqual(expected_json, lan_function.get_json())

    def test_lan_function_detail(self):
        lan_function = viom_elcm.LANFunction(
            4, function_enable=False, vlan_id=123, bandwidth=50,
            rate_limit=100, sriov=True)
        expected_json = {
            '@FunctionIdx': 4,
            'LANFunction': {
                'FunctionEnable': False,
                'BootProtocol': 'None',
                'BootPriority': 1,
                'VLANId': 123,
                'Bandwidth': 50,
                'RateLimit': 100,
                'SRIOV': True,
            }
        }
        self.assertEqual(expected_json, lan_function.get_json())

    def test_fcoe_function(self):
        fcoe_function = viom_elcm.FCoEFunction(
            1, boot=VIOMTableTestCase._sample_fc_boot())
        expected_json = {
            '@FunctionIdx': 1,
            'FCoEFunction': {
                'FunctionEnable': True,
                'BootProtocol': 'FC',
                'BootPriority': 1,
            },
        }
        expected_json['FCoEFunction'].update(
            VIOMTableTestCase._sample_fc_boot_json())
        self.assertEqual(expected_json, fcoe_function.get_json())

    def test_fcoe_function_boot_priority(self):
        fcoe_function = viom_elcm.FCoEFunction(
            2, boot=VIOMTableTestCase._sample_fc_boot(boot_prio=3))
        expected_json = {
            '@FunctionIdx': 2,
            'FCoEFunction': {
                'FunctionEnable': True,
                'BootProtocol': 'FC',
                'BootPriority': 3,
            },
        }
        expected_json['FCoEFunction'].update(
            VIOMTableTestCase._sample_fc_boot_json())
        self.assertEqual(expected_json, fcoe_function.get_json())

    def test_fcoe_function_virtualize_wwn(self):
        fcoe_function = viom_elcm.FCoEFunction(
            3, boot=VIOMTableTestCase._sample_fc_boot(),
            use_virtual_addresses=True,
            wwnn='11:22:33:44:55', wwpn='66:77:88:99:00')
        expected_json = {
            '@FunctionIdx': 3,
            'FCoEFunction': {
                'FunctionEnable': True,
                'BootProtocol': 'FC',
                'BootPriority': 1,
                'UseVirtualAddresses': True,
                'VirtualAddress': {
                    'WWNN': '11:22:33:44:55',
                    'WWPN': '66:77:88:99:00'
                },
            },
        }
        expected_json['FCoEFunction'].update(
            VIOMTableTestCase._sample_fc_boot_json())
        self.assertEqual(expected_json, fcoe_function.get_json())

    def test_fcoe_function_virtualize_mac(self):
        fcoe_function = viom_elcm.FCoEFunction(
            4, boot=VIOMTableTestCase._sample_fc_boot(),
            use_virtual_addresses=True, mac='aa:bb:cc:dd:ee')
        expected_json = {
            '@FunctionIdx': 4,
            'FCoEFunction': {
                'FunctionEnable': True,
                'BootProtocol': 'FC',
                'BootPriority': 1,
                'UseVirtualAddresses': True,
                'VirtualAddress': {
                    'MAC': 'aa:bb:cc:dd:ee'
                },
            },
        }
        expected_json['FCoEFunction'].update(
            VIOMTableTestCase._sample_fc_boot_json())
        self.assertEqual(expected_json, fcoe_function.get_json())

    def test_fcoe_function_detail(self):
        fcoe_function = viom_elcm.FCoEFunction(
            5, boot=VIOMTableTestCase._sample_fc_boot(),
            function_enable=False, vlan_id=123, bandwidth=50, rate_limit=100,
            sriov=True)
        expected_json = {
            '@FunctionIdx': 5,
            'FCoEFunction': {
                'FunctionEnable': False,
                'BootProtocol': 'FC',
                'BootPriority': 1,
                'VLANId': 123,
                'Bandwidth': 50,
                'RateLimit': 100,
                'SRIOV': True,
            },
        }
        expected_json['FCoEFunction'].update(
            VIOMTableTestCase._sample_fc_boot_json())
        self.assertEqual(expected_json, fcoe_function.get_json())

    def test_iscsi_function(self):
        iscsi_function = viom_elcm.ISCSIFunction(
            1, boot=VIOMTableTestCase._sample_iscsi_boot())
        expected_json = {
            '@FunctionIdx': 1,
            'ISCSIFunction': {
                'FunctionEnable': True,
                'BootProtocol': 'ISCSI',
                'BootPriority': 1,
            },
        }
        expected_json['ISCSIFunction'].update(
            VIOMTableTestCase._sample_iscsi_boot_json())
        self.assertEqual(expected_json, iscsi_function.get_json())

    def test_iscsi_function_boot_priority(self):
        iscsi_function = viom_elcm.ISCSIFunction(
            2, boot=VIOMTableTestCase._sample_iscsi_boot(boot_prio=3))
        expected_json = {
            '@FunctionIdx': 2,
            'ISCSIFunction': {
                'FunctionEnable': True,
                'BootProtocol': 'ISCSI',
                'BootPriority': 3,
            },
        }
        expected_json['ISCSIFunction'].update(
            VIOMTableTestCase._sample_iscsi_boot_json())
        self.assertEqual(expected_json, iscsi_function.get_json())

    def test_iscsi_functon_virtulaize_mac(self):
        iscsi_function = viom_elcm.ISCSIFunction(
            3, boot=VIOMTableTestCase._sample_iscsi_boot(),
            use_virtual_addresses=True, mac='12:34:56:78:90')
        expected_json = {
            '@FunctionIdx': 3,
            'ISCSIFunction': {
                'FunctionEnable': True,
                'BootProtocol': 'ISCSI',
                'BootPriority': 1,
                'UseVirtualAddresses': True,
                'VirtualAddress': {
                    'MAC': '12:34:56:78:90'
                },
            },
        }
        expected_json['ISCSIFunction'].update(
            VIOMTableTestCase._sample_iscsi_boot_json())
        self.assertEqual(expected_json, iscsi_function.get_json())

    def test_iscsi_functon_detail(self):
        iscsi_function = viom_elcm.ISCSIFunction(
            4, boot=VIOMTableTestCase._sample_iscsi_boot(),
            function_enable=False, vlan_id=123, bandwidth=50, rate_limit=100,
            sriov=True)
        expected_json = {
            '@FunctionIdx': 4,
            'ISCSIFunction': {
                'FunctionEnable': False,
                'BootProtocol': 'ISCSI',
                'BootPriority': 1,
                'VLANId': 123,
                'Bandwidth': 50,
                'RateLimit': 100,
                'SRIOV': True,
            },
        }
        expected_json['ISCSIFunction'].update(
            VIOMTableTestCase._sample_iscsi_boot_json())
        self.assertEqual(expected_json, iscsi_function.get_json())

    def test_none_boot(self):
        none_boot = viom_elcm.NoneBoot()
        self.assertEqual({}, none_boot.get_json())

    def test_pxe_boot(self):
        pxe_boot = viom_elcm.PXEBoot()
        self.assertEqual({}, pxe_boot.get_json())

    def test_fc_boot(self):
        fc_boot = viom_elcm.FCBoot()
        fc_boot.add_target(VIOMTableTestCase._sample_fc_target())
        expected_json = {
            'FCBootEnvironment': {
                'FCTargets': {
                    'FCTarget': [VIOMTableTestCase._sample_fc_target_json()]
                },
                'FCLinkSpeed': 'auto',
                'FCTopology': 'auto_loop',
            }
        }
        self.assertEqual(expected_json, fc_boot.get_json())

    def test_fc_boot_detail(self):
        fc_boot = viom_elcm.FCBoot(
            link_speed='auto',
            topology='auto_PtP',
            boot_enable=False)
        fc_boot.add_target(VIOMTableTestCase._sample_fc_target())
        expected_json = {
            'FCBootEnvironment': {
                'FCTargets': {
                    'FCTarget': [VIOMTableTestCase._sample_fc_target_json()]
                },
                'FCLinkSpeed': 'auto',
                'FCTopology': 'auto_PtP',
                'SANBootEnable': False,
            }
        }
        self.assertEqual(expected_json, fc_boot.get_json())

    def test_fc_boot_mulit_targets(self):
        fc_boot = viom_elcm.FCBoot()
        fc_boot.add_target(VIOMTableTestCase._sample_fc_target())
        fc_boot.add_target(
            VIOMTableTestCase._sample_fc_target('aa:bb:cc:dd:ee:ff'))
        expected_json = {
            'FCBootEnvironment': {
                'FCTargets': {
                    'FCTarget': [
                        VIOMTableTestCase._sample_fc_target_json(),
                        VIOMTableTestCase._sample_fc_target_json(
                            target_idx=2, wwpn='aa:bb:cc:dd:ee:ff')
                    ]
                },
                'FCLinkSpeed': 'auto',
                'FCTopology': 'auto_loop',
            }
        }
        self.assertEqual(expected_json, fc_boot.get_json())

    def test_fc_target(self):
        fc_target = viom_elcm.FCTarget('11:22:33:44:55')
        expected_json = {
            '@FCTargetIdx': 1,
            'TargetWWPN': '11:22:33:44:55',
            'TargetLUN': 0,
        }
        self.assertEqual(expected_json, fc_target.get_json())

    def test_fc_target_lun(self):
        fc_target = viom_elcm.FCTarget('11:22:33:44:55', lun=1)
        expected_json = {
            '@FCTargetIdx': 1,
            'TargetWWPN': '11:22:33:44:55',
            'TargetLUN': 1,
        }
        self.assertEqual(expected_json, fc_target.get_json())

    def test_iscsi_boot(self):
        iscsi_boot = viom_elcm.ISCSIBoot(
            VIOMTableTestCase._sample_iscsi_initiator(),
            VIOMTableTestCase._sample_iscsi_target())
        expected_json = {
            'ISCSIBootEnvironment': {
                'ISCSIInitiator':
                    VIOMTableTestCase._sample_iscsi_initiator_json(),
                'ISCSITarget':
                    VIOMTableTestCase._sample_iscsi_target_json()
            }
        }
        self.assertEqual(expected_json, iscsi_boot.get_json())

    def test_iscsi_initiator(self):
        iscsi_initiator = viom_elcm.ISCSIInitiator(
            iqn='iqn-2017-04.com.fujitsu:001',
            ip='192.168.1.11',
            subnet='255.255.255.0',
            gateway='192.168.1.1'
        )
        expected_json = {
            'DHCPUsage': False,
            'Name': 'iqn-2017-04.com.fujitsu:001',
            'IPv4Address': '192.168.1.11',
            'SubnetMask': '255.255.255.0',
            'GatewayIPv4Address': '192.168.1.1',
            'VLANId': 0,
        }
        self.assertEqual(expected_json, iscsi_initiator.get_json())

    def test_iscsi_initiator_vlan(self):
        iscsi_initiator = viom_elcm.ISCSIInitiator(
            iqn='iqn-2017-04.com.fujitsu:001',
            ip='192.168.1.11',
            subnet='255.255.255.0',
            gateway='192.168.1.1',
            vlan_id=123
        )
        expected_json = {
            'DHCPUsage': False,
            'Name': 'iqn-2017-04.com.fujitsu:001',
            'IPv4Address': '192.168.1.11',
            'SubnetMask': '255.255.255.0',
            'GatewayIPv4Address': '192.168.1.1',
            'VLANId': 123,
        }
        self.assertEqual(expected_json, iscsi_initiator.get_json())

    def test_iscsi_initiator_dhcp(self):
        iscsi_initiator = viom_elcm.ISCSIInitiator(
            dhcp_usage=True,
            iqn='iqn-2017-04.com.fujitsu:001',
            ip='192.168.1.11',
            subnet='255.255.255.0',
        )
        expected_json = {
            'DHCPUsage': True,
            'Name': 'iqn-2017-04.com.fujitsu:001',
        }
        self.assertEqual(expected_json, iscsi_initiator.get_json())

    def test_iscsi_target_no_auth(self):
        iscsi_target = viom_elcm.ISCSITarget(
            iqn='iqn-2017-04.com.fujitsu:101',
            ip='192.168.2.22')
        expected_json = {
            'DHCPUsage': False,
            'Name': 'iqn-2017-04.com.fujitsu:101',
            'IPv4Address': '192.168.2.22',
            'PortNumber': 3260,
            'BootLUN': 0,
            'AuthenticationMethod': 'None',
        }
        self.assertEqual(expected_json, iscsi_target.get_json())

    def test_iscsi_target_chap(self):
        iscsi_target = viom_elcm.ISCSITarget(
            iqn='iqn-2017-04.com.fujitsu:101',
            ip='192.168.2.22',
            port=12345,
            lun=3,
            auth_method='CHAP',
            chap_user='chap_user',
            chap_secret='chap_secret')
        expected_json = {
            'DHCPUsage': False,
            'Name': 'iqn-2017-04.com.fujitsu:101',
            'IPv4Address': '192.168.2.22',
            'PortNumber': 12345,
            'BootLUN': 3,
            'AuthenticationMethod': 'CHAP',
            'ChapUserName': 'chap_user',
            'ChapSecret': 'chap_secret',
        }
        self.assertEqual(expected_json, iscsi_target.get_json())

    def test_iscsi_target_mutualchap(self):
        iscsi_target = viom_elcm.ISCSITarget(
            iqn='iqn-2017-04.com.fujitsu:101',
            ip='192.168.2.22',
            auth_method='MutualCHAP',
            chap_user='chap_user',
            chap_secret='chap_secret',
            mutual_chap_secret='chap_secret_second')
        expected_json = {
            'DHCPUsage': False,
            'Name': 'iqn-2017-04.com.fujitsu:101',
            'IPv4Address': '192.168.2.22',
            'PortNumber': 3260,
            'BootLUN': 0,
            'AuthenticationMethod': 'MutualCHAP',
            'ChapUserName': 'chap_user',
            'ChapSecret': 'chap_secret',
            'MutualChapSecret': 'chap_secret_second',
        }
        self.assertEqual(expected_json, iscsi_target.get_json())

    def test_iscsi_target_dhcp(self):
        iscsi_target = viom_elcm.ISCSITarget(
            dhcp_usage=True,
            iqn='iqn-2017-04.com.fujitsu:101',
            ip='192.168.2.22',
            auth_method='CHAP',
            chap_user='chap_user',
            chap_secret='chap_secret')
        expected_json = {
            'DHCPUsage': True,
            'AuthenticationMethod': 'CHAP',
            'ChapUserName': 'chap_user',
            'ChapSecret': 'chap_secret',
        }
        self.assertEqual(expected_json, iscsi_target.get_json())



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
Test class for snmp module.
"""

from unittest import mock

from pysnmp import error as snmp_error
from pysnmp import hlapi as pysnmp
import testtools

from scciclient.irmc import snmp


class IRMCSnmpTestCase(testtools.TestCase):
    """Tests for SNMP module

    Unit Test Cases for getting information via snmp module
    """

    def setUp(self):
        super(IRMCSnmpTestCase, self).setUp()

    def test_get_irmc_firmware_version(self):
        snmp_client = mock.Mock()
        snmp_client.get.side_effect = ['iRMC S4', '7.82F']
        cmd1 = snmp.BMC_NAME_OID
        cmd2 = snmp.IRMC_FW_VERSION_OID
        actual_out = snmp.get_irmc_firmware_version(snmp_client)
        self.assertEqual('iRMC S4-7.82F', actual_out)
        snmp_client.get.assert_has_calls([mock.call(cmd1),
                                          mock.call(cmd2)])

    def test_get_irmc_firmware_version_BMC_only(self):
        snmp_client = mock.Mock()
        snmp_client.get.side_effect = ['iRMC S4', '']
        cmd1 = snmp.BMC_NAME_OID
        cmd2 = snmp.IRMC_FW_VERSION_OID
        actual_out = snmp.get_irmc_firmware_version(snmp_client)
        self.assertEqual('iRMC S4', actual_out)
        snmp_client.get.assert_has_calls([mock.call(cmd1),
                                          mock.call(cmd2)])

    def test_get_irmc_firmware_version_FW_only(self):
        snmp_client = mock.Mock()
        snmp_client.get.side_effect = ['', '7.82F']
        cmd1 = snmp.BMC_NAME_OID
        cmd2 = snmp.IRMC_FW_VERSION_OID
        actual_out = snmp.get_irmc_firmware_version(snmp_client)
        self.assertEqual('7.82F', actual_out)
        snmp_client.get.assert_has_calls([mock.call(cmd1),
                                          mock.call(cmd2)])

    def test_get_irmc_firmware_version_blank(self):
        snmp_client = mock.Mock()
        snmp_client.get.side_effect = ['', '']
        cmd1 = snmp.BMC_NAME_OID
        cmd2 = snmp.IRMC_FW_VERSION_OID
        actual_out = snmp.get_irmc_firmware_version(snmp_client)
        self.assertEqual('', actual_out)
        snmp_client.get.assert_has_calls([mock.call(cmd1),
                                          mock.call(cmd2)])

    def test_get_irmc_firmware_version_exception(self):
        snmp_client = mock.Mock()
        snmp_client.get.side_effect = snmp.SNMPFailure('Error')
        cmd1 = snmp.BMC_NAME_OID
        e = self.assertRaises(snmp.SNMPIRMCFirmwareFailure,
                              snmp.get_irmc_firmware_version,
                              snmp_client)
        snmp_client.get.assert_has_calls([mock.call(cmd1)])
        self.assertEqual('SNMP operation \'GET IRMC FIRMWARE VERSION\''
                         ' failed: Error', str(e))

    def test_get_bios_firmware_version(self):
        snmp_client = mock.Mock()
        snmp_client.return_value = 'V4.6.5.4 R1.15.0 for D3099-B1x'
        snmp_client.get.return_value = 'V4.6.5.4 R1.15.0 for D3099-B1x'
        cmd = snmp.BIOS_FW_VERSION_OID
        actual_out = snmp.get_bios_firmware_version(snmp_client)
        self.assertEqual('V4.6.5.4 R1.15.0 for D3099-B1x', actual_out)
        snmp_client.get.assert_called_once_with(cmd)

    def test_get_bios_firmware_version_exception(self):
        snmp_client = mock.Mock()
        snmp_client.get.side_effect = snmp.SNMPFailure('Error')
        cmd = snmp.BIOS_FW_VERSION_OID
        e = self.assertRaises(snmp.SNMPBIOSFirmwareFailure,
                              snmp.get_bios_firmware_version,
                              snmp_client)
        snmp_client.get.assert_called_once_with(cmd)
        self.assertEqual('SNMP operation \'GET BIOS FIRMWARE VERSION\''
                         ' failed: Error', str(e))

    def test_get_server_model(self):
        snmp_client = mock.Mock()
        snmp_client.return_value = 'TX2540M1F5'
        snmp_client.get.return_value = 'TX2540M1F5'
        cmd = snmp.SERVER_MODEL_OID
        actual_out = snmp.get_server_model(snmp_client)
        self.assertEqual('TX2540M1F5', actual_out)
        snmp_client.get.assert_called_once_with(cmd)

    def test_get_server_model_exception(self):
        snmp_client = mock.Mock()
        snmp_client.get.side_effect = snmp.SNMPFailure('Error')
        cmd = snmp.SERVER_MODEL_OID
        e = self.assertRaises(snmp.SNMPServerModelFailure,
                              snmp.get_server_model,
                              snmp_client)
        snmp_client.get.assert_called_once_with(cmd)
        self.assertEqual('SNMP operation \'GET SERVER MODEL\''
                         ' failed: Error', str(e))


class SNMPClientTestCase(testtools.TestCase):
    def setUp(self):
        super(SNMPClientTestCase, self).setUp()
        self.address = '1.2.3.4'
        self.port = '6700'
        self.oid = 'oid'
        self.value = 'value'

    @mock.patch.object(pysnmp, 'SnmpEngine', authspec=True)
    def test___init__(self, mock_snmpengine):
        client = snmp.SNMPClient(self.address, self.port, snmp.SNMP_V1)
        mock_snmpengine.assert_called_once_with()
        self.assertEqual(self.address, client.address)
        self.assertEqual(self.port, client.port)
        self.assertEqual(snmp.SNMP_V1, client.version)
        self.assertIsNone(client.read_community)
        self.assertIsNone(client.write_community)
        self.assertNotIn('user', client.__dict__)
        self.assertEqual(mock_snmpengine.return_value, client.snmp_engine)

    @mock.patch.object(pysnmp, 'CommunityData', autospec=True)
    def test__get_auth_v1_read(self, mock_community):
        client = snmp.SNMPClient(self.address, self.port, snmp.SNMP_V1,
                                 read_community='public',
                                 write_community='private')
        client._get_auth()
        mock_community.assert_called_once_with(client.read_community,
                                               mpModel=0)

    @mock.patch.object(pysnmp, 'CommunityData', autospec=True)
    def test__get_auth_v1_write(self, mock_community):
        client = snmp.SNMPClient(self.address, self.port, snmp.SNMP_V1,
                                 read_community='public',
                                 write_community='private')
        client._get_auth(write_mode=True)
        mock_community.assert_called_once_with(client.write_community,
                                               mpModel=0)

    @mock.patch.object(pysnmp, 'CommunityData', autospec=True)
    def test__get_auth_v2c(self, mock_community):
        client = snmp.SNMPClient(self.address, self.port, snmp.SNMP_V2C)
        client._get_auth()
        mock_community.assert_called_once_with(client.read_community,
                                               mpModel=1)

    @mock.patch.object(pysnmp, 'UsmUserData', autospec=True)
    def test__get_auth_v3(self, mock_user):
        client = snmp.SNMPClient(self.address, self.port, snmp.SNMP_V3)
        client._get_auth()
        mock_user.assert_called_once_with(client.user,
                                          authKey=client.auth_key,
                                          authProtocol=client.auth_proto,
                                          privKey=client.priv_key,
                                          privProtocol=client.priv_proto)

    @mock.patch.object(pysnmp, 'UdpTransportTarget', autospec=True)
    def test__get_transport(self, mock_transport):
        client = snmp.SNMPClient(self.address, self.port, snmp.SNMP_V3)
        client._get_transport()
        mock_transport.assert_called_once_with((client.address, client.port))

    @mock.patch.object(pysnmp, 'UdpTransportTarget', autospec=True)
    def test__get_transport_err(self, mock_transport):
        mock_transport.side_effect = snmp_error.PySnmpError
        client = snmp.SNMPClient(self.address, self.port, snmp.SNMP_V3)
        self.assertRaises(snmp_error.PySnmpError, client._get_transport)
        mock_transport.assert_called_once_with((client.address, client.port))

    @mock.patch.object(pysnmp, 'ContextData', authspec=True)
    def test__get_context(self, mock_context):
        client = snmp.SNMPClient(self.address, self.port, snmp.SNMP_V1)
        client._get_context()
        mock_context.assert_called_once_with(contextEngineId=None,
                                             contextName='')

    @mock.patch.object(pysnmp, 'getCmd', authspec=True)
    @mock.patch.object(snmp.SNMPClient, '_get_transport', authspec=True)
    @mock.patch.object(snmp.SNMPClient, '_get_context', authspec=True)
    @mock.patch.object(snmp.SNMPClient, '_get_auth', authspec=True)
    def test_get(self, mock_auth, mock_context, mock_transport,
                 mock_getcmd):
        var_bind = (self.oid, self.value)
        mock_getcmd.return_value = iter([("", None, 0, [var_bind])])
        client = snmp.SNMPClient(self.address, self.port, snmp.SNMP_V3)
        val = client.get(self.oid)
        self.assertEqual(var_bind[1], val)
        self.assertEqual(1, mock_getcmd.call_count)

    @mock.patch.object(pysnmp, 'nextCmd', authspec=True)
    @mock.patch.object(snmp.SNMPClient, '_get_transport', authspec=True)
    @mock.patch.object(snmp.SNMPClient, '_get_context', authspec=True)
    @mock.patch.object(snmp.SNMPClient, '_get_auth', authspec=True)
    def test_get_next(self, mock_auth, mock_context, mock_transport,
                      mock_nextcmd):
        var_bind = (self.oid, self.value)
        mock_nextcmd.return_value = iter([("", None, 0, [var_bind]),
                                          ("", None, 0, [var_bind])])
        client = snmp.SNMPClient(self.address, self.port, snmp.SNMP_V3)
        val = client.get_next(self.oid)
        self.assertEqual([self.value, self.value], val)
        self.assertEqual(1, mock_nextcmd.call_count)

    @mock.patch.object(pysnmp, 'getCmd', authspec=True)
    @mock.patch.object(snmp.SNMPClient, '_get_transport', authspec=True)
    @mock.patch.object(snmp.SNMPClient, '_get_context', authspec=True)
    @mock.patch.object(snmp.SNMPClient, '_get_auth', authspec=True)
    def test_get_err_transport(self, mock_auth, mock_context, mock_transport,
                               mock_getcmd):
        mock_transport.side_effect = snmp_error.PySnmpError
        var_bind = (self.oid, self.value)
        mock_getcmd.return_value = iter([("engine error", None, 0,
                                         [var_bind])])
        client = snmp.SNMPClient(self.address, self.port, snmp.SNMP_V3)
        self.assertRaises(snmp.SNMPFailure, client.get, self.oid)
        self.assertFalse(mock_getcmd.called)

    @mock.patch.object(pysnmp, 'nextCmd', authspec=True)
    @mock.patch.object(snmp.SNMPClient, '_get_transport', authspec=True)
    @mock.patch.object(snmp.SNMPClient, '_get_context', authspec=True)
    @mock.patch.object(snmp.SNMPClient, '_get_auth', authspec=True)
    def test_get_next_err_transport(self, mock_auth, mock_context,
                                    mock_transport, mock_nextcmd):
        mock_transport.side_effect = snmp_error.PySnmpError
        var_bind = (self.oid, self.value)
        mock_nextcmd.return_value = iter([("engine error", None, 0,
                                         [var_bind])])
        client = snmp.SNMPClient(self.address, self.port, snmp.SNMP_V3)
        self.assertRaises(snmp.SNMPFailure, client.get_next, self.oid)
        self.assertFalse(mock_nextcmd.called)

    @mock.patch.object(pysnmp, 'getCmd', authspec=True)
    @mock.patch.object(snmp.SNMPClient, '_get_transport', authspec=True)
    @mock.patch.object(snmp.SNMPClient, '_get_context', authspec=True)
    @mock.patch.object(snmp.SNMPClient, '_get_auth', authspec=True)
    def test_get_err_engine(self, mock_auth, mock_context, mock_transport,
                            mock_getcmd):
        var_bind = (self.oid, self.value)
        mock_getcmd.return_value = iter([("engine error", None, 0,
                                         [var_bind])])
        client = snmp.SNMPClient(self.address, self.port, snmp.SNMP_V3)
        self.assertRaises(snmp.SNMPFailure, client.get, self.oid)
        self.assertEqual(1, mock_getcmd.call_count)

    @mock.patch.object(pysnmp, 'nextCmd', authspec=True)
    @mock.patch.object(snmp.SNMPClient, '_get_transport', authspec=True)
    @mock.patch.object(snmp.SNMPClient, '_get_context', authspec=True)
    @mock.patch.object(snmp.SNMPClient, '_get_auth', authspec=True)
    def test_get_next_err_engine(self, mock_auth, mock_context, mock_transport,
                                 mock_nextcmd):
        var_bind = (self.oid, self.value)
        mock_nextcmd.return_value = iter([("engine error", None, 0,
                                         [var_bind])])
        client = snmp.SNMPClient(self.address, self.port, snmp.SNMP_V3)
        self.assertRaises(snmp.SNMPFailure, client.get_next, self.oid)
        self.assertEqual(1, mock_nextcmd.call_count)

    @mock.patch.object(pysnmp, 'setCmd', authspec=True)
    @mock.patch.object(snmp.SNMPClient, '_get_transport', authspec=True)
    @mock.patch.object(snmp.SNMPClient, '_get_context', authspec=True)
    @mock.patch.object(snmp.SNMPClient, '_get_auth', authspec=True)
    def test_set(self, mock_auth, mock_context, mock_transport,
                 mock_setcmd):
        var_bind = (self.oid, self.value)
        mock_setcmd.return_value = iter([("", None, 0, [var_bind])])
        client = snmp.SNMPClient(self.address, self.port, snmp.SNMP_V3)
        client.set(self.oid, self.value)
        self.assertEqual(1, mock_setcmd.call_count)

    @mock.patch.object(pysnmp, 'setCmd', authspec=True)
    @mock.patch.object(snmp.SNMPClient, '_get_transport', authspec=True)
    @mock.patch.object(snmp.SNMPClient, '_get_context', authspec=True)
    @mock.patch.object(snmp.SNMPClient, '_get_auth', authspec=True)
    def test_set_err_transport(self, mock_auth, mock_context, mock_transport,
                               mock_setcmd):
        mock_transport.side_effect = snmp_error.PySnmpError
        client = snmp.SNMPClient(self.address, self.port, snmp.SNMP_V3)
        self.assertRaises(snmp.SNMPFailure,
                          client.set, self.oid, self.value)
        self.assertFalse(mock_setcmd.called)

    @mock.patch.object(pysnmp, 'setCmd', authspec=True)
    @mock.patch.object(snmp.SNMPClient, '_get_transport', authspec=True)
    @mock.patch.object(snmp.SNMPClient, '_get_context', authspec=True)
    @mock.patch.object(snmp.SNMPClient, '_get_auth', authspec=True)
    def test_set_err_engine(self, mock_auth, mock_context, mock_transport,
                            mock_setcmd):
        var_bind = (self.oid, self.value)
        mock_setcmd.return_value = iter([("engine error", None, 0,
                                         [var_bind])])
        client = snmp.SNMPClient(self.address, self.port, snmp.SNMP_V3)
        self.assertRaises(snmp.SNMPFailure,
                          client.set, self.oid, self.value)
        self.assertEqual(1, mock_setcmd.call_count)
